/********************************************************************************
*                                                                               *
*                      F X   M O V I E   M A N A G E R                          *
*                                                                               *
*********************************************************************************
* Copyright(C) 2008,2015 by Luigi Ziviani <ziviani_luigi@yahoo.it>.             *
* All Rights Reserved.                            version 6.7                   *
*********************************************************************************
* Id: fxtxtview.cpp,                              2008/01/01                    *
*********************************************************************************
* This file is part of FXMOVIEMANAGER                                           *
*                                                                               *
* FXMOVIEMANAGER is free software; you can redistribute it and/or modify        *
* it under the terms of the GNU General Public License as published by          *
* the Free Software Foundation; either version 3 of the License, or             *
* (at your option) any later version.                                           *
*                                                                               *
* FXMOVIEMANAGER is distributed in the hope that it will be useful,             *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 *
* GNU General Public License for more details.                                  *
*                                                                               *
* You should have received a copy of the GNU General Public License             *
* along with this program; if not, write to the Free Software                   *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA     * 
********************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <string.h>

#ifndef WIN32
#include <unistd.h>
#endif

#include <fx.h>
#include <fxkeys.h>

#include "config.h"

#ifdef HAVE_LIBFOX_1_6
 #include <FXGIFIcon.h>
 #include <FXPNGIcon.h>
#endif

#include "fxtxtview.h"
#include "commands.h"
#include "icons.h"

// Map
FXDEFMAP(TextEncryptor) TextEncryptorMap[]={
  FXMAPFUNC(SEL_COMMAND,           TextEncryptor::ID_ABOUT,            TextEncryptor::onCmdAbout),
  FXMAPFUNC(SEL_COMMAND,           TextEncryptor::ID_HELP,             TextEncryptor::onCmdHelp),
  FXMAPFUNC(SEL_COMMAND,           TextEncryptor::ID_OPEN,             TextEncryptor::onCmdOpen),
  FXMAPFUNC(SEL_COMMAND,           TextEncryptor::ID_NEW,              TextEncryptor::onCmdNew),
  FXMAPFUNC(SEL_COMMAND,           TextEncryptor::ID_SAVE,             TextEncryptor::onCmdSave),
  FXMAPFUNC(SEL_UPDATE,            TextEncryptor::ID_SAVE,             TextEncryptor::onUpdSave),
  FXMAPFUNC(SEL_COMMAND,           TextEncryptor::ID_SAVEAS,           TextEncryptor::onCmdSaveAs),
  FXMAPFUNC(SEL_COMMAND,           TextEncryptor::ID_SETKEY,           TextEncryptor::onCmdSetKey),
  FXMAPFUNC(SEL_COMMAND,           TextEncryptor::ID_ENCRYPT2,         TextEncryptor::onCmdEncrypt2),
  FXMAPFUNC(SEL_COMMAND,           TextEncryptor::ID_DECRYPT2,         TextEncryptor::onCmdDecrypt2),
  FXMAPFUNC(SEL_COMMAND,           TextEncryptor::ID_FIND,             TextEncryptor::onCmdFind),
  FXMAPFUNC(SEL_UPDATE,            TextEncryptor::ID_FIND,             TextEncryptor::onUpdMenu),
  FXMAPFUNC(SEL_COMMAND,           TextEncryptor::ID_FIND_AGAIN,       TextEncryptor::onCmdFind),
  FXMAPFUNC(SEL_UPDATE,            TextEncryptor::ID_FIND_AGAIN,       TextEncryptor::onCmdFind),
  FXMAPFUNC(SEL_COMMAND,           TextEncryptor::ID_QUIT,             TextEncryptor::onCmdQuit),
  FXMAPFUNC(SEL_COMMAND,           TextEncryptor::ID_PRINT,            TextEncryptor::onCmdPrint),
  FXMAPFUNC(SEL_UPDATE,            TextEncryptor::ID_PRINT,            TextEncryptor::onUpdMenu),
  FXMAPFUNC(SEL_COMMAND,           TextEncryptor::ID_FONT,             TextEncryptor::onCmdFont),
  FXMAPFUNC(SEL_UPDATE,            TextEncryptor::ID_TITLE,            TextEncryptor::onUpdTitle),
  FXMAPFUNC(SEL_CHANGED,           TextEncryptor::ID_TEXT,             TextEncryptor::onTextChanged),
  FXMAPFUNC(SEL_INSERTED,          TextEncryptor::ID_TEXT,             TextEncryptor::onTextInserted),
  FXMAPFUNC(SEL_REPLACED,          TextEncryptor::ID_TEXT,             TextEncryptor::onTextReplaced),
  FXMAPFUNC(SEL_DELETED,           TextEncryptor::ID_TEXT,             TextEncryptor::onTextDeleted),
  FXMAPFUNC(SEL_RIGHTBUTTONRELEASE,TextEncryptor::ID_TEXT,             TextEncryptor::onTextRightMouse),
  };


// Object implementation
FXIMPLEMENT(TextEncryptor,FXMainWindow,TextEncryptorMap,ARRAYNUMBER(TextEncryptorMap))


// Make some windows
TextEncryptor::TextEncryptor(FXApp* a)
    :FXMainWindow(a,"For X Text Viewer ..and editor",NULL,NULL,DECOR_ALL,0,0,600,500,0,0){


    // set icon on the main window
    logoicon=new FXPNGIcon(getApp(),fxtv64);
    logominiicon=new FXPNGIcon(getApp(),fxtv16);
    setIcon(logoicon);
    setMiniIcon(logominiicon);

    // Create toolbar icons
    abouticon=new FXGIFIcon(getApp(),b_about);
    allicon=new FXGIFIcon(getApp(),b_all);
    buildicon=new FXGIFIcon(getApp(),b_build);
    build2icon=new FXGIFIcon(getApp(),b_build2);
    conficon=new FXGIFIcon(getApp(),b_conf);
    copyicon=new FXGIFIcon(getApp(),b_copy);
    cuticon=new FXGIFIcon(getApp(),b_cut);
    deleteicon=new FXGIFIcon(getApp(),b_delete);
    exiticon=new FXGIFIcon(getApp(),b_exit);
    fontsicon=new FXGIFIcon(getApp(),b_fonts);
    newicon=new FXGIFIcon(getApp(),b_new);
    openicon=new FXGIFIcon(getApp(),b_open);
    pasteicon=new FXGIFIcon(getApp(),b_paste);
    printicon=new FXGIFIcon(getApp(),b_print);
    redoicon=new FXGIFIcon(getApp(),b_redo);
    saveicon=new FXGIFIcon(getApp(),b_save);
    saveasicon=new FXGIFIcon(getApp(),b_save_as);
    searchicon=new FXGIFIcon(getApp(),b_search);
    searchagainicon=new FXGIFIcon(getApp(),b_search_again);
    undoicon=new FXGIFIcon(getApp(),b_undo);

    if(!readConfig()){
	printf("Configuration not loaded!");
    }

    // Make main window; set myself as the target
    setTarget(this);
    // Set the message identifier for this data target
    setSelector(ID_TITLE);

    // Site where to dock
    FXDockSite *docksite=new FXDockSite(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X);

    // Make menu bar
    FXToolBarShell* dragshell=new FXToolBarShell(this,FRAME_RAISED);
    menubar=new FXMenuBar(docksite,dragshell,LAYOUT_DOCK_NEXT|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|FRAME_RAISED);
    new FXToolBarGrip(menubar,menubar,FXMenuBar::ID_TOOLBARGRIP,TOOLBARGRIP_DOUBLE);

    // Tool bar
    FXToolBarShell* dragshell1=new FXToolBarShell(this,FRAME_RAISED);
    toolbar=new FXToolBar(docksite,dragshell1,LAYOUT_DOCK_NEXT|LAYOUT_SIDE_TOP|LAYOUT_FILL_X|FRAME_RAISED);
    new FXToolBarGrip(toolbar,toolbar,FXToolBar::ID_TOOLBARGRIP,TOOLBARGRIP_DOUBLE);
    int style=BUTTON_TOOLBAR;


  // mainframe
  globalframe= new FXVerticalFrame(this,LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 0,0,0,0,0,0);

  // TabContents
  tabcontents=new FXHorizontalFrame(globalframe,LAYOUT_SIDE_TOP|FRAME_NONE|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH);
  // TabSwitcher
  tabbook=new FXTabBook(tabcontents,this,ID_PANEL,PACK_UNIFORM_WIDTH|PACK_UNIFORM_HEIGHT|LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_RIGHT);

  // First item is file operation options
  tab1=new FXTabItem(tabbook,"&File operations \tSwitch to first panel",NULL);
  tab1frame=new FXHorizontalFrame(tabbook,FRAME_NONE);

  packer = new FXPacker(tab1frame,LAYOUT_LEFT|LAYOUT_FILL_Y);
  button_open=new FXButton(packer,menu_op+"\tCtrl-O\t"+menu_op+" (Ctrl-O)",openicon,this,ID_OPEN,BUTTON_DEFAULT|JUSTIFY_NORMAL|ICON_BEFORE_TEXT|LAYOUT_CENTER_X,0,0,0,0,6,6);
  button_new=new FXButton(packer,menu_ne+"\tCtrl-N\t"+menu_ne+" (Ctrl-N)",newicon,this,ID_NEW,BUTTON_DEFAULT|JUSTIFY_NORMAL|ICON_BEFORE_TEXT|LAYOUT_CENTER_X,0,0,0,0,6,6);

  new FXVerticalSeparator(tab1frame,SEPARATOR_GROOVE|LAYOUT_SIDE_LEFT|LAYOUT_FILL_Y);
  packer = new FXPacker(tab1frame,LAYOUT_LEFT|LAYOUT_FILL_Y);
  button_save=new FXButton(packer,menu_ss+"\tCtrl-S\t"+menu_ss+" (Ctrl-S)",saveicon,this,ID_SAVE,BUTTON_DEFAULT|JUSTIFY_NORMAL|ICON_BEFORE_TEXT|LAYOUT_CENTER_X,0,0,0,0,6,6);
  button_saveas=new FXButton(packer,menu_sa+"\tCtrl-W\t"+menu_sa+" (Ctrl-W)",saveasicon,this,ID_SAVEAS,BUTTON_DEFAULT|JUSTIFY_NORMAL|ICON_BEFORE_TEXT|LAYOUT_CENTER_X,0,0,0,0,6,6);

  new FXVerticalSeparator(tab1frame,SEPARATOR_GROOVE|LAYOUT_SIDE_LEFT|LAYOUT_FILL_Y);
  packer = new FXPacker(tab1frame,LAYOUT_LEFT|LAYOUT_FILL_Y);
  button_print=new FXButton(packer,menu_pd+"\tCtrl-P\t"+menu_pd+" (Ctrl-P)",printicon,this,ID_PRINT,BUTTON_DEFAULT|JUSTIFY_NORMAL|ICON_ABOVE_TEXT|LAYOUT_CENTER_X|LAYOUT_CENTER_Y,0,0,0,0,6,6);

  new FXVerticalSeparator(tab1frame,SEPARATOR_GROOVE|LAYOUT_SIDE_LEFT|LAYOUT_FILL_Y);
  packer = new FXPacker(tab1frame,LAYOUT_LEFT|LAYOUT_FILL_Y);
  button_quit=new FXButton(packer,menu_qu+"\tCtrl-Q\t"+menu_qu+" (Ctrl-Q)",exiticon,this,ID_QUIT,BUTTON_DEFAULT|JUSTIFY_NORMAL|ICON_ABOVE_TEXT|LAYOUT_CENTER_X|LAYOUT_CENTER_Y,0,0,0,0,6,6);


  // Second item is file operation options
  tab2=new FXTabItem(tabbook,"&Editing \tSwitch to second panel",NULL);
  tab2frame=new FXHorizontalFrame(tabbook,FRAME_NONE);

  packer = new FXPacker(tab2frame,LAYOUT_LEFT|LAYOUT_FILL_Y);
  button_undo=new FXButton(packer,"Undo\tCtrl-Z\tUndo last change in the document. (Ctrl-Z)",undoicon,&undolist,FXUndoList::ID_UNDO,BUTTON_DEFAULT|JUSTIFY_NORMAL|ICON_BEFORE_TEXT|LAYOUT_CENTER_X,0,0,0,0,6,6);
  button_redo=new FXButton(packer,"Redo\tCtrl-Shift-Z\tRedo last undo in the document. (Ctrl-Shift-Z)",redoicon,&undolist,FXUndoList::ID_REDO,BUTTON_DEFAULT|JUSTIFY_NORMAL|ICON_BEFORE_TEXT|LAYOUT_CENTER_X,0,0,0,0,6,6);

    // Make contents
    contents=new FXHorizontalFrame(globalframe,FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 0,0,0,0, 0,0);
    // Make editor window
    editor=new FXText(contents,this,ID_TEXT,LAYOUT_FILL_X|LAYOUT_FILL_Y);

  new FXVerticalSeparator(tab2frame,SEPARATOR_GROOVE|LAYOUT_SIDE_LEFT|LAYOUT_FILL_Y);
  packer = new FXPacker(tab2frame,LAYOUT_LEFT|LAYOUT_FILL_Y);
  button_cut=new FXButton(packer,menu_cu+"\tCtrl-X\tCut text in the document",cuticon,editor,FXText::ID_CUT_SEL,BUTTON_DEFAULT|JUSTIFY_NORMAL|ICON_BEFORE_TEXT|LAYOUT_CENTER_X,0,0,0,0,6,6);
  button_copy=new FXButton(packer,menu_co+"\tCtrl-C\tCopy text in the document",copyicon,editor,FXText::ID_COPY_SEL,BUTTON_DEFAULT|JUSTIFY_NORMAL|ICON_BEFORE_TEXT|LAYOUT_CENTER_X,0,0,0,0,6,6);

  packer = new FXPacker(tab2frame,LAYOUT_LEFT|LAYOUT_FILL_Y);
  button_paste=new FXButton(packer,menu_pa+"\tCtrl-V\tPaste text in the document",pasteicon,editor,FXText::ID_PASTE_SEL,BUTTON_DEFAULT|JUSTIFY_NORMAL|ICON_ABOVE_TEXT|LAYOUT_CENTER_X|LAYOUT_CENTER_Y,0,0,0,0,6,6);

  new FXVerticalSeparator(tab2frame,SEPARATOR_GROOVE|LAYOUT_SIDE_LEFT|LAYOUT_FILL_Y);
  packer = new FXPacker(tab2frame,LAYOUT_LEFT|LAYOUT_FILL_Y);
  button_delete=new FXButton(packer,menu_de+"\t\tDelete selected text in the document",deleteicon,editor,FXText::ID_DELETE_SEL,BUTTON_DEFAULT|JUSTIFY_NORMAL|ICON_ABOVE_TEXT|LAYOUT_CENTER_X|LAYOUT_CENTER_Y,0,0,0,0,6,6);


  // Third item is file operation options
  tab3=new FXTabItem(tabbook,"&Utilities \tSwitch to third panel",NULL);
  tab3frame=new FXHorizontalFrame(tabbook,FRAME_NONE);

  packer = new FXPacker(tab3frame,LAYOUT_LEFT|LAYOUT_FILL_Y);
  button_find=new FXButton(packer,menu_ff+"\tCtrl-F\t"+menu_ff+" (Ctrl-F)",searchicon,this,ID_FIND,BUTTON_DEFAULT|JUSTIFY_NORMAL|ICON_BEFORE_TEXT|LAYOUT_CENTER_X,0,0,0,0,6,6);
  button_finda=new FXButton(packer,menu_fa+"\tCtrl-G\t"+menu_fa+" (Ctrl-G)",searchagainicon,this,ID_FIND_AGAIN,BUTTON_DEFAULT|JUSTIFY_NORMAL|ICON_BEFORE_TEXT|LAYOUT_CENTER_X,0,0,0,0,6,6);

  new FXVerticalSeparator(tab3frame,SEPARATOR_GROOVE|LAYOUT_SIDE_LEFT|LAYOUT_FILL_Y);
  packer = new FXPacker(tab3frame,LAYOUT_LEFT|LAYOUT_FILL_Y);
  button_setkey=new FXButton(packer,menu_cf+" Key\tCtrl-K\t"+menu_cf+" Key (Ctrl-K)",conficon,this,ID_SETKEY,BUTTON_DEFAULT|JUSTIFY_NORMAL|ICON_ABOVE_TEXT|LAYOUT_CENTER_X|LAYOUT_CENTER_Y,0,0,0,0,6,6);

  packer = new FXPacker(tab3frame,LAYOUT_LEFT|LAYOUT_FILL_Y);
  button_encrypt=new FXButton(packer,menu_en+" Encrypt\tCtrl-E\t"+menu_en+" Vigenère Encrypt (Ctrl-E)",buildicon,this,ID_ENCRYPT2,BUTTON_DEFAULT|JUSTIFY_NORMAL|ICON_BEFORE_TEXT|LAYOUT_CENTER_X,0,0,0,0,6,6);
  button_decrypt=new FXButton(packer,menu_en+" Decrypt\tCtrl-D\t"+menu_en+" Vigenère Decrypt (Ctrl-D)",build2icon,this,ID_DECRYPT2,BUTTON_DEFAULT|JUSTIFY_NORMAL|ICON_BEFORE_TEXT|LAYOUT_CENTER_X,0,0,0,0,6,6);


  // Fourth item is file operation options
  tab4=new FXTabItem(tabbook,"&View options \tSwitch to fourth panel",NULL);
  tab4frame=new FXHorizontalFrame(tabbook,FRAME_NONE);

    // Status bar
    statusbar=new FXStatusBar(this,FRAME_RAISED|LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|STATUSBAR_WITH_DRAGCORNER);

  packer = new FXPacker(tab4frame,LAYOUT_LEFT|LAYOUT_FILL_Y);;
  check_menubar=new FXMenuCheck(packer,"Menubar\t\tDisplay or hide menubar.",menubar,FXWindow::ID_TOGGLESHOWN);
  check_toolbar=new FXMenuCheck(packer,"Toolbar\t\tDisplay or hide toolbar.",toolbar,FXWindow::ID_TOGGLESHOWN);
  check_statusbar=new FXMenuCheck(packer,"Status bar\t\tDisplay or hide status bar.",statusbar,FXWindow::ID_TOGGLESHOWN);

  new FXVerticalSeparator(tab4frame,SEPARATOR_GROOVE|LAYOUT_SIDE_LEFT|LAYOUT_FILL_Y);
  packer = new FXPacker(tab4frame,LAYOUT_LEFT|LAYOUT_FILL_Y);
  button_font=new FXButton(packer,menu_fo+"\tCtrl-T\t"+menu_fo+" (Ctrl-T)",fontsicon,this,ID_FONT,BUTTON_DEFAULT|JUSTIFY_NORMAL|ICON_ABOVE_TEXT|LAYOUT_CENTER_X|LAYOUT_CENTER_Y,0,0,0,0,6,6);

  new FXVerticalSeparator(tab4frame,SEPARATOR_GROOVE|LAYOUT_SIDE_LEFT|LAYOUT_FILL_Y);
  packer = new FXPacker(tab4frame,LAYOUT_LEFT|LAYOUT_FILL_Y);
  button_about=new FXButton(packer,"About FXTxtView\tF1\tAbout FXTxtView.(F1)",abouticon,this,ID_ABOUT,BUTTON_DEFAULT|JUSTIFY_NORMAL|ICON_BEFORE_TEXT|LAYOUT_CENTER_X,0,0,0,0,6,6);
  button_help=new FXButton(packer,"Using FXTxtView\t\tUsing FXTxtView.",abouticon,this,ID_HELP,BUTTON_DEFAULT|JUSTIFY_NORMAL|ICON_BEFORE_TEXT|LAYOUT_CENTER_X,0,0,0,0,6,6);
  
    // File menu
    filemenu=new FXMenuPane(this);
    title_file=new FXMenuTitle(menubar,"&File",NULL,filemenu);

    // Edit menu
    editmenu=new FXMenuPane(this);
    title_edit=new FXMenuTitle(menubar,"&Edit",NULL,editmenu);

    // Search Menu
    searchmenu=new FXMenuPane(this);
    title_search=new FXMenuTitle(menubar,"&Search",NULL,searchmenu);

    // Preferences Menu
    prefsmenu=new FXMenuPane(this);
    title_pref=new FXMenuTitle(menubar,"&Preferences",NULL,prefsmenu);

    // Crypy Menu
    cryptmenu=new FXMenuPane(this);
    title_crypt=new FXMenuTitle(menubar,"&Utilities",NULL,cryptmenu);

    // Help menu
    helpmenu=new FXMenuPane(this);
    title_help=new FXMenuTitle(menubar,"&Help",NULL,helpmenu);

    // Show Undo-Redo count in status bar
    undoredoblock=new FXHorizontalFrame(statusbar,LAYOUT_RIGHT|LAYOUT_CENTER_Y,0,0,0,0, 0,0,0,0);
    new FXLabel(undoredoblock,"  Undo:",NULL,LAYOUT_CENTER_Y);
    FXTextField* undocount=new FXTextField(undoredoblock,4,&undolist,FXUndoList::ID_UNDO_COUNT,TEXTFIELD_READONLY|FRAME_SUNKEN|JUSTIFY_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,2,2,1,1);
    undocount->setBackColor(statusbar->getBackColor());
    new FXLabel(undoredoblock,"  Redo:",NULL,LAYOUT_CENTER_Y);
    FXTextField* redocount=new FXTextField(undoredoblock,4,&undolist,FXUndoList::ID_REDO_COUNT,TEXTFIELD_READONLY|FRAME_SUNKEN|JUSTIFY_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,2,2,1,1);
    redocount->setBackColor(statusbar->getBackColor());
    // Show column number in status bar
    new FXTextField(statusbar,4,editor,FXText::ID_CURSOR_COLUMN,FRAME_SUNKEN|LAYOUT_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,0,0,0,0);
    new FXLabel(statusbar,"Col:",NULL,LAYOUT_RIGHT|LAYOUT_CENTER_Y);
    // Show line number in status bar
    new FXTextField(statusbar,4,editor,FXText::ID_CURSOR_ROW,FRAME_SUNKEN|LAYOUT_RIGHT|LAYOUT_CENTER_Y,0,0,0,0,0,0,0,0);
    new FXLabel(statusbar,"Line:",NULL,LAYOUT_RIGHT|LAYOUT_CENTER_Y);

    // Toolbar button: File
    button_open=new FXButton(toolbar,"\t"+menu_op+"\t"+menu_op+" (Ctrl-O)",openicon,this,ID_OPEN,style|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
    button_save=new FXButton(toolbar,"\t"+menu_ss+"\t"+menu_ss+" (Ctrl-S)",saveicon,this,ID_SAVE,style|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
    button_saveas=new FXButton(toolbar,"\t"+menu_sa+"\t"+menu_sa+" (Ctrl-W)",saveasicon,this,ID_SAVEAS,style|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);

    // Toolbar button: Print
    new FXFrame(toolbar,LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,10,20);
    button_print=new FXButton(toolbar,"\t"+menu_pd+"\t"+menu_pd+" (Ctrl-P)",printicon,this,ID_PRINT,style|BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
    button_font=new FXButton(toolbar,"\t"+menu_fo+"\t"+menu_fo+" (Ctrl-T)",fontsicon,this,ID_FONT,style|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);

    // Toolbar buttons: Undo-Redo
    new FXFrame(toolbar,LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,10,20);
    button_undo=new FXButton(toolbar,"\tUndo last change\tUndo last change in the document. (Ctrl-U)",undoicon,&undolist,FXUndoList::ID_UNDO,style|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
    button_redo=new FXButton(toolbar,"\tRedo last undo\tRedo last undo in the document. (Ctrl-R)",redoicon,&undolist,FXUndoList::ID_REDO,style|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);

    // Toolbar buttons: Search
    new FXFrame(toolbar,LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,10,20);
    button_find=new FXButton(toolbar,"\t"+menu_ff+"\t"+menu_ff+" (Ctrl-F)",searchicon,this,ID_FIND,style|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
    button_finda=new FXButton(toolbar,"\t"+menu_fa+"\t"+menu_fa+" (Ctrl-G)",searchagainicon,this,ID_FIND_AGAIN,style|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);

    // Toolbar buttons: Encrypt
    new FXFrame(toolbar,LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,10,20);
    button_setkey =new FXButton(toolbar,"\t"+menu_cf+" Key\t"+menu_cf+" Key (Ctrl-K)",conficon,this,ID_SETKEY,BUTTON_DEFAULT|JUSTIFY_NORMAL|ICON_BEFORE_TEXT|LAYOUT_CENTER_X|LAYOUT_CENTER_Y,0,0,0,0,6,6);
    button_encrypt=new FXButton(toolbar,"\t"+menu_en+" Encrypt\t"+menu_en+" Vigenère Encrypt (Ctrl-E)",buildicon,this,ID_ENCRYPT2,style|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);
    button_decrypt=new FXButton(toolbar,"\t"+menu_en+" Decrypt\t"+menu_en+" Vigenère Decrypt (Ctrl-D)",build2icon,this,ID_DECRYPT2,style|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);

    // Toolbar button: Quit
    new FXFrame(toolbar,LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,10,20);
    button_quit=new FXButton(toolbar,"\t"+menu_qu+"\t"+menu_qu+" (Ctrl-Q)",exiticon,this,ID_QUIT,style|FRAME_RAISED|LAYOUT_TOP|LAYOUT_LEFT);


    // File Menu entries
    //new FXMenuCommand(filemenu,"&Open file\tCtl-O\tOpen document file. (Ctrl-O)",openicon,this,ID_OPEN);
    cmd_new   =new FXMenuCommand(filemenu,menu_ne+"\tCtrl-N\t"+menu_ne+" (Ctrl-N)",newicon,this,ID_NEW);
    cmd_open  =new FXMenuCommand(filemenu,menu_op+"\tCtrl-O\t"+menu_op+" (Ctrl-O)",openicon,this,ID_OPEN);
    cmd_save  =new FXMenuCommand(filemenu,menu_ss+"\tCtrl-S\t"+menu_ss+" (Ctrl-S)",saveicon,this,ID_SAVE);
    cmd_savea =new FXMenuCommand(filemenu,menu_sa+"\tCtrl-W\t"+menu_sa+" (Ctrl-W)",saveasicon,this,ID_SAVEAS);
    cmd_print =new FXMenuCommand(filemenu,menu_pd+"\tCtrl-P\t"+menu_pd+" (Ctrl-P)",printicon,this,ID_PRINT);
    new FXMenuSeparator(filemenu);
    cmd_quit  =new FXMenuCommand(filemenu,menu_qu+"\tCtrl-Q\t"+menu_qu+" (Ctrl-Q)",exiticon,this,ID_QUIT);

    // Edit Menu entries
    cmd_undo  =new FXMenuCommand(editmenu,"Undo\tCtrl-Z\tUndo text in the document. (Ctrl-Z)",undoicon,&undolist,FXUndoList::ID_UNDO);
    cmd_redo  =new FXMenuCommand(editmenu,"Redo\tCtl-Shift-Z\tRedo text in the document. (Ctl-Shift-Z)",redoicon,&undolist,FXUndoList::ID_REDO);
    new FXMenuSeparator(editmenu);
    cmd_cut   =new FXMenuCommand(editmenu,menu_cu+"\tCtrl-X\t"+menu_cu+" (Ctrl-X)",cuticon,editor,FXText::ID_CUT_SEL);
    cmd_copy  =new FXMenuCommand(editmenu,menu_co+"\tCtrl-C\t"+menu_co+" (Ctrl-C)",copyicon,editor,FXText::ID_COPY_SEL);
    cmd_paste =new FXMenuCommand(editmenu,menu_pa+"\tCtrl-V\t"+menu_pa+" (Ctrl-V)",pasteicon,editor,FXText::ID_PASTE_SEL);
    cmd_delete=new FXMenuCommand(editmenu,menu_de+"\t\t"+menu_de,deleteicon,editor,FXText::ID_DELETE_SEL);

    // Right mouse popup
    popupmenu =new FXMenuPane(this);
    cmd_pcut  =new FXMenuCommand(popupmenu,menu_cu+" (Ctrl-X)",cuticon,editor,FXText::ID_CUT_SEL);
    cmd_pcopy =new FXMenuCommand(popupmenu,menu_co+" (Ctrl-C)",copyicon,editor,FXText::ID_COPY_SEL);
    cmd_ppaste=new FXMenuCommand(popupmenu,menu_pa+" (Ctrl-V)",pasteicon,editor,FXText::ID_PASTE_SEL);
    cmd_all   =new FXMenuCommand(popupmenu,menu_al+" (Ctrl-A)",allicon,editor,FXText::ID_SELECT_ALL);
    cmd_pdelete=new FXMenuCommand(popupmenu,menu_de,deleteicon,editor,FXText::ID_DELETE_SEL);


    // Search Menu entries
    cmd_find  =new FXMenuCommand(searchmenu,menu_ff+"\tCtrl-F\t"+menu_ff+" (Ctrl-F)",searchicon,this,ID_FIND);
    cmd_finda =new FXMenuCommand(searchmenu,menu_fa+"\tCtrl-G\t"+menu_fa+" (Ctrl-G)",searchagainicon,this,ID_FIND_AGAIN);

    // Preferences menu
    check_tabbook=new FXMenuCheck(prefsmenu,"Tabbook\t\tDisplay or hide Tabbook.",tabbook,FXWindow::ID_TOGGLESHOWN);
    check_toolbar=new FXMenuCheck(prefsmenu,"Toolbar\t\tDisplay or hide toolbar.",toolbar,FXWindow::ID_TOGGLESHOWN);
    check_statusbar=new FXMenuCheck(prefsmenu,"Status bar\t\tDisplay or hide status bar.",statusbar,FXWindow::ID_TOGGLESHOWN);
    new FXMenuSeparator(prefsmenu);
    cmd_font  =new FXMenuCommand(prefsmenu,menu_fo+"\tCtrl-T\t"+menu_fo+" (Ctrl-T)",fontsicon,this,ID_FONT);

    // Crypt menu
    cmd_key=new FXMenuCommand(cryptmenu,menu_cf+" Key\tCtrl-K\t"+menu_cf+" Key (Ctrl-K)",conficon,this,ID_SETKEY);
    new FXMenuSeparator(cryptmenu);
    cmd_encrypt=new FXMenuCommand(cryptmenu,menu_en+" Encrypt\tCtrl-E\t"+menu_en+" Vigenère Encrypt (Ctrl-E)",buildicon,this,ID_ENCRYPT2);
    cmd_decrypt=new FXMenuCommand(cryptmenu,menu_en+" Decrypt\tCtrl-D\t"+menu_en+" Vigenère Decrypt (Ctrl-D)",build2icon,this,ID_DECRYPT2);

    // Help Menu entries
    cmd_about =new FXMenuCommand(helpmenu,"About FXTxtView\tF1\tAbout FXTxtView.(F1)",abouticon,this,ID_ABOUT,0);
    cmd_help  =new FXMenuCommand(helpmenu,"Using FXTxtView\t\tUsing FXTxtView.",abouticon,this,ID_HELP,0);


    // Build a matrix dialog box
    keydialog = new fxmmKeyDialog(this);

    // Make a tool tip
    new FXToolTip(getApp(),0);

    // Add some alternative accelerators
    if(getAccelTable()){

       getAccelTable()->addAccel(MKUINT(KEY_F1,0),this,FXSEL(SEL_COMMAND,ID_ABOUT));

	//*******************************************************************
	// Remaind to add the accelerators in case menucommands are wiped out
	//*******************************************************************
       //getAccelTable()->addAccel(MKUINT(KEY_n,CONTROLMASK),this,FXSEL(SEL_COMMAND,ID_NEW));
       //getAccelTable()->addAccel(MKUINT(KEY_o,CONTROLMASK),this,FXSEL(SEL_COMMAND,ID_OPEN));
       //getAccelTable()->addAccel(MKUINT(KEY_s,CONTROLMASK),this,FXSEL(SEL_COMMAND,ID_SAVE));
       //getAccelTable()->addAccel(MKUINT(KEY_w,CONTROLMASK),this,FXSEL(SEL_COMMAND,ID_SAVEAS));

       //getAccelTable()->addAccel(MKUINT(KEY_p,CONTROLMASK),this,FXSEL(SEL_COMMAND,ID_PRINT));
       //getAccelTable()->addAccel(MKUINT(KEY_t,CONTROLMASK),this,FXSEL(SEL_COMMAND,ID_FONT));
       //getAccelTable()->addAccel(MKUINT(KEY_q,CONTROLMASK),this,FXSEL(SEL_COMMAND,ID_QUIT));

       //getAccelTable()->addAccel(MKUINT(KEY_f,CONTROLMASK),this,FXSEL(SEL_COMMAND,ID_FIND));
       //getAccelTable()->addAccel(MKUINT(KEY_g,CONTROLMASK),this,FXSEL(SEL_COMMAND,ID_FIND_AGAIN));

       //getAccelTable()->addAccel(MKUINT(KEY_Z,CONTROLMASK),&undolist,FXSEL(SEL_COMMAND,FXUndoList::ID_UNDO));
       //getAccelTable()->addAccel(MKUINT(KEY_Z,CONTROLMASK|SHIFTMASK),&undolist,FXSEL(SEL_COMMAND,FXUndoList::ID_REDO));

       //getAccelTable()->addAccel(MKUINT(KEY_x,CONTROLMASK),this,FXSEL(SEL_COMMAND,FXText::ID_CUT_SEL));
       //getAccelTable()->addAccel(MKUINT(KEY_c,CONTROLMASK),this,FXSEL(SEL_COMMAND,FXText::ID_COPY_SEL));
       //getAccelTable()->addAccel(MKUINT(KEY_v,CONTROLMASK),this,FXSEL(SEL_COMMAND,FXText::ID_PASTE_SEL));
       //getAccelTable()->addAccel(MKUINT(KEY_a,CONTROLMASK),this,FXSEL(SEL_COMMAND,FXText::ID_SELECT_ALL));
    }

    // Initialize 
    filename="";
    password="";
           /* ccccccNccccccNccccccNccccNccccNccccNccccNccccccNccccccNccccccN */
    alfabeto="AaBbCc0DdEeFf1GgHhIi2JjKk3LlMm4NnOo5PpQq6RrSsTt7UuVvWw8XxYyZz9";
    filenameset=false;
    keyfileset=false;
    stripcr=true;
    undolist.mark();
}

// Clean up
TextEncryptor::~TextEncryptor(){
//    delete bigp;
    delete allicon;
    delete buildicon;
    delete conficon;
    delete copyicon;
    delete cuticon;
    delete deleteicon;
    delete fontsicon;
    delete exiticon;
    delete newicon;
    delete openicon;
    delete pasteicon;
    delete printicon;
    delete redoicon;
    delete saveicon;
    delete searchicon;
    delete searchagainicon;
    delete undoicon;

    delete filemenu;
    delete editmenu;
    delete searchmenu;
    delete prefsmenu;
    delete cryptmenu;
    delete helpmenu;
    delete popupmenu;
    /*****************************/
}



// Is it modified
FXbool TextEncryptor::isModified() const {
  return !undolist.marked();
}


// Save file
FXbool TextEncryptor::saveFile(const FXString& file){
  FXuint size,n;
  FXchar *text;
  FILE *fp;

  FXTRACE((100,"saveFile(%s)\n",file.text()));

  // Open file
  fp=fopen(file.text(),"w");
  if(!fp){
    FXMessageBox::error(this,MBOX_OK,"Error Saving File","Unable to open file: %s",file.text());
    return false;
  }

  // Get size
  size=editor->getLength();

  // Alloc buffer
#ifdef HAVE_LIBFOX_1_6
  if(!FXMALLOC(&text,FXchar,size+1)){
#endif
#ifdef HAVE_LIBFOX_1_7
  if(!allocElms(text,size+1)){
#endif
    FXMessageBox::error(this,MBOX_OK,"Error Saving File","File is too big: %s",file.text());
    fclose(fp);
    return false;
  }

  // Set wait cursor
  getApp()->beginWaitCursor();

  // Get text from editor
  editor->getText(text,size);

  // Write the file
  n=fwrite(text,1,size,fp);

  // Ditch buffer
#ifdef HAVE_LIBFOX_1_6
  FXFREE(&text);
#endif
#ifdef HAVE_LIBFOX_1_7
  freeElms(text);
#endif

  // Kill wait cursor
  getApp()->endWaitCursor();

  // Close file
  fclose(fp);

  // Were we able to write it all?
  if(n!=size){
    FXMessageBox::error(this,MBOX_OK,"Error Saving File","File: %s truncated.",file.text());
    return false;
  }

  // Set stuff
  editor->setEditable();
  filenameset=true;
  filename=file;
  undolist.mark();
  return true;
  }


// Load file
FXbool TextEncryptor::loadFile(const FXString& file){
  FXFile textfile(file,FXFile::Reading);
  FXuint size, n, i, j, c;
  FXchar *text;

  FXTRACE((100,"loadFile(%s)\n",file.text()));

  // Opened file?
  if(!textfile.isOpen()){
    FXMessageBox::error(this,MBOX_OK,tr("Error Loading File"),tr("Unable to open file: %s"),file.text());
    return false;
    }

  // Get file size
  size=textfile.size();

  // Make buffer to load file
#ifdef HAVE_LIBFOX_1_6
  if(!FXMALLOC(&text,FXchar,size)){
#endif
#ifdef HAVE_LIBFOX_1_7
  if(!allocElms(text,size)){
#endif
    FXMessageBox::error(this,MBOX_OK,"Error Loading File","File is too big: %s (%d bytes)",file.text(),size);
    return false;
    }

  // Set wait cursor
  getApp()->beginWaitCursor();

  // Read the file
  n=textfile.readBlock(text,size);
  if(n<0){
#ifdef HAVE_LIBFOX_1_6
  FXFREE(&text);
#endif
#ifdef HAVE_LIBFOX_1_7
  freeElms(text);
#endif
    FXMessageBox::error(this,MBOX_OK,tr("Error Loading File"),tr("Unable to read file: %s"),file.text());
    return false;
    }

  // Strip carriage returns
  if(stripcr){
    for(i=j=0; j<n; j++){
      c=text[j];
      if(c!='\r'){
        text[i++]=c;
        }
      }
    n=i;
    }

  // Set text
  editor->setText(text,n);
#ifdef HAVE_LIBFOX_1_6
  FXFREE(&text);
#endif
#ifdef HAVE_LIBFOX_1_7
  freeElms(text);
#endif

  // Kill wait cursor
  getApp()->endWaitCursor();

  // Set stuff
  editor->setEditable();
  filenameset=true;
  filename=file;

  // Clear undo records
  undolist.clear();

  // Mark undo state as clean (saved)
  undolist.mark();

  return true;
}


// Set Password
FXString TextEncryptor::setKey(FXString& password){

  if(keydialog->execute()){
     password=keydialog->getPassword();
     if (comparecase (password, "")!=0){
         keyfileset=true;
		 //printf("pw: %s\n", password.text());
     }
     else{
         password="";
         keyfileset=false;
     }
  }

  return password;
}


// Set Password
long TextEncryptor::onCmdSetKey(FXObject*,FXSelector,void*){

  if(keyfileset){
     FXint c=0;
     FXint pwlen=password.length();
     FXString pwhelp;
     for(int i=0; i<pwlen; i++){
        pwhelp.append("* ");
        c++;
        if(c==60){
          c=0;
          pwhelp.append("\n");
        }
     }
     if(MBOX_CLICKED_NO==FXMessageBox::question(this,MBOX_YES_NO,"Overwrite Secret Key","Overwrite existing key: \n%s?",pwhelp.text())) return 1;
  }

  setKey(password);

  return 1;
}


// Dencrypt
FXbool TextEncryptor::decryptFile2(){

  FXString chiave = password;
  if(!keyfileset)
     setKey(chiave);

  if(!keyfileset)
     return false;

  password = chiave;
  //printf("k: %s\n", chiave.text());

  FXuint size;
  FXchar *text;

  size=editor->getLength();

  // Make buffer to load file
#ifdef HAVE_LIBFOX_1_6
  if(!FXMALLOC(&text,FXchar,size)){
#endif
#ifdef HAVE_LIBFOX_1_7
  if(!allocElms(text,size)){
#endif
    //FXMessageBox::error(this,MBOX_OK,"Error Loading File","File is too big: %s (%d bytes)",file.text(),size);
    FXMessageBox::error(this,MBOX_OK,"Error Working on File","File is too big");
    return false;
  }
  getApp()->beginWaitCursor();
  editor->getText(text,size);

  int j=0;
  FXint totale, resto;

  for(int i=0; i<size; i++) {
	FXint t2i=alfabeto.find(text[i]);
    //printf("char matrice: %c, %i\n", c, t2i);

	if( t2i != -1 ){
		FXint c2i=alfabeto.find(chiave[j]);
        //printf("char chiave: %c, %i\n", chiave[j], c2i);
        j++;
        if (j==chiave.length())
           j = 0;

        totale=t2i-c2i+62;
        if (totale<62){
           text[i]=alfabeto[totale];
        } else {
           resto=totale-62;
           text[i]=alfabeto[resto];
        }
	}
  }

  editor->setText(text,size);
#ifdef HAVE_LIBFOX_1_6
  FXFREE(&text);
#endif
#ifdef HAVE_LIBFOX_1_7
  freeElms(text);
#endif
  getApp()->endWaitCursor();

  return true;
}

// Decrypt
long TextEncryptor::onCmdDecrypt2(FXObject* sender,FXSelector sel,void* ptr){
  if(!decryptFile2()){
    FXMessageBox::error(this,MBOX_OK,"Error Encrypting File","Unable to encrypt file: %s",filename.text());
  }
  return 1;
}


// Encrypt
FXbool TextEncryptor::encryptFile2(){

  FXString chiave = password;
  if(!keyfileset)
     setKey(chiave);

  if(!keyfileset)
     return false;

  password = chiave;
  //printf("pw: %s\n", password.text());

  FXuint size;
  FXchar *text;

  size=editor->getLength();

  // Make buffer to load file
#ifdef HAVE_LIBFOX_1_6
  if(!FXMALLOC(&text,FXchar,size)){
#endif
#ifdef HAVE_LIBFOX_1_7
  if(!allocElms(text,size)){
#endif
    //FXMessageBox::error(this,MBOX_OK,"Error Loading File","File is too big: %s (%d bytes)",file.text(),size);
    FXMessageBox::error(this,MBOX_OK,"Error Working on File","File is too big");
    return false;
  }
  getApp()->beginWaitCursor();
  editor->getText(text,size);

  int j=0;
  FXint totale, resto;

  for(int i=0; i<size; i++) {
	FXint t2i=alfabeto.find(text[i]);
    //printf("char matrice: %c, %i\n", c, c2i);

	if( t2i != -1 ){
		FXint c2i=alfabeto.find(chiave[j]);
        //printf("char chiave: %c, %i\n", chiave[j], c1i);
        j++;
        if (j==chiave.length())
           j = 0;

        totale=c2i+t2i;
        if (totale<62){
           text[i]=alfabeto[totale];
        } else {
           resto=totale-62;
           text[i]=alfabeto[resto];
        }
	}
  }

  editor->setText(text,size);
#ifdef HAVE_LIBFOX_1_6
  FXFREE(&text);
#endif
#ifdef HAVE_LIBFOX_1_7
  freeElms(text);
#endif
  getApp()->endWaitCursor();

  return true;
}


// Encrypt
long TextEncryptor::onCmdEncrypt2(FXObject* sender,FXSelector sel,void* ptr){
  if(!encryptFile2()){
    FXMessageBox::error(this,MBOX_OK,"Error Encrypting File","Unable to encrypt file: %s",filename.text());
  }
  return 1;
}


// Save
long TextEncryptor::onCmdSave(FXObject* sender,FXSelector sel,void* ptr){
  if(!filenameset) return onCmdSaveAs(sender,sel,ptr);
  if(!saveFile(filename)){
    FXMessageBox::error(this,MBOX_OK,"Error Saving File","Unable to save file: %s",filename.text());
  }
  return 1;
}


// Save As
long TextEncryptor::onCmdSaveAs(FXObject*,FXSelector,void*){
  FXFileDialog savedialog(this,"Save Document");
  FXString file=filename;
  savedialog.setSelectMode(SELECTFILE_ANY);
  savedialog.setFilename(file);
  if(savedialog.execute()){
    file=savedialog.getFilename();
    if(FXStat::exists(file)){
      if(MBOX_CLICKED_NO==FXMessageBox::question(this,MBOX_YES_NO,"Overwrite Document","Overwrite existing document: %s?",file.text())) return 1;
      }
      if(!saveFile(file)){
        FXMessageBox::error(this,MBOX_OK,"Error Saving File","Unable to save file: %s",filename.text());
      }
    }
  return 1;
  }


// Save changes, ask for new filename
FXbool TextEncryptor::saveChanges(){
    FXuint answer=FXMessageBox::question(this,MBOX_YES_NO_CANCEL,"Unsaved Document","Save %s to file?",filename.text());
    if(answer==MBOX_CLICKED_CANCEL) return false;
    if(answer==MBOX_CLICKED_NO) return true;
    if(answer==MBOX_CLICKED_YES){
      FXString file=filename;
      if(!filenameset){
        FXFileDialog savedialog(this,"Save Document");
        savedialog.setSelectMode(SELECTFILE_ANY);
        savedialog.setFilename(file);
        if(!savedialog.execute()) return false;
        file=savedialog.getFilename();
        if(FXStat::exists(file)){
          if(MBOX_CLICKED_NO==FXMessageBox::question(this,MBOX_YES_NO,"Overwrite Document","Overwrite existing document: %s?",file.text())) return false;
        }
      }
      if(!saveFile(file)){
        FXMessageBox::error(this,MBOX_OK,"Error Saving File","Unable to save file: %s",filename.text());
      }
  return true;
    }
  return false;
  }


// New
long TextEncryptor::onCmdNew(FXObject*,FXSelector,void*){
      // ask for saving changes on the current document
    if(isModified()){
      if(!saveChanges()) 
        return 1;
    }
    editor->setText("");
    editor->setEditable();

    filename="newdocument.txt";
    filenameset=false;

    // Clear undo records
    undolist.clear();
    // Mark undo state as clean (saved)
    undolist.mark();

    return 1;
}


// Open
long TextEncryptor::onCmdOpen(FXObject*,FXSelector,void*){
    if(isModified()){
      if(!saveChanges())
        return 1;
    }
    const FXchar patterns[]=
            "All Files (*)"
            "\nText Files (*.txt)"
            "\nHTML Header Files (*.html)";

    FXFileDialog open(this,"Open Document");
    open.setSelectMode(SELECTFILE_EXISTING);
    open.setPatternList(patterns);
    open.setDirectory(FXPath::directory(filename));
    if(open.execute())
    {
        filename=open.getFilename();
        loadFile(filename);
        editor->setFocus();
    }
    return 1;
}

void TextEncryptor::setPrinter(){
  FXPrintDialog dlg(this,"Print File");
  if(dlg.execute()){
    dlg.getPrinter(printer);
    FXTRACE((100,"Printer = %s\n",printer.name.text()));
  }
  return;
}

// Print the text
long TextEncryptor::onCmdPrint(FXObject*,FXSelector,void*){

    setPrinter();

    FXDCPrint prt(getApp());

    FXuint size, n;
    FXchar *text;

    // Get size
    size=editor->getLength();

  // Alloc buffer
#ifdef HAVE_LIBFOX_1_6
    if(!FXMALLOC(&text,FXchar,size+1)){
#endif
#ifdef HAVE_LIBFOX_1_7
    if(!allocElms(text,size+1)){
#endif
      FXMessageBox::error(this,MBOX_OK,"Error Printing File","File is too big: %s",filename.text());
      return false;
    }
    // Set wait cursor
    getApp()->beginWaitCursor();

    // Get text from editor
    editor->getText(text,size);

    prt.beginPrint(printer);

    prt.beginPage(1);

    prt.setForeground( FXRGB(0,0,0) );

    prt.setFont( textfont );
    prt.drawText(300,300, text, size );
    
    prt.endPage();
    prt.endPrint();

    // Ditch buffer
#ifdef HAVE_LIBFOX_1_6
    FXFREE(&text);
#endif
#ifdef HAVE_LIBFOX_1_7
    freeElms(text);
#endif

    // Kill wait cursor
    getApp()->endWaitCursor();

  return 1;
}


// Save configuration when quitting
void TextEncryptor::saveConfig(){
    // Write new window size and position back to registry
    getApp()->reg().writeIntEntry("FXTxtView","xpos",getX());
    getApp()->reg().writeIntEntry("FXTxtView","ypos",getY());
    getApp()->reg().writeIntEntry("FXTxtView","width",getWidth());
    getApp()->reg().writeIntEntry("FXTxtView","height",getHeight());

    // Tabbook status
    if(tabbook->shown())
        getApp()->reg().writeBoolEntry("FXTxtView","showtabbook",true);
    else
        getApp()->reg().writeBoolEntry("FXTxtView","showtabbook",false);

    // Menubar status
    if(menubar->shown())
        getApp()->reg().writeBoolEntry("FXTxtView","showmenubar",true);
    else
        getApp()->reg().writeBoolEntry("FXTxtView","showmenubar",false);

    // Toolbar status
    if(toolbar->shown())
        getApp()->reg().writeBoolEntry("FXTxtView","showtoolbar",true);
    else
        getApp()->reg().writeBoolEntry("FXTxtView","showtoolbar",false);

    // Status bar status
    if(statusbar->shown())
        getApp()->reg().writeBoolEntry("FXTxtView","showstatusbar",true);
    else
        getApp()->reg().writeBoolEntry("FXTxtView","showstatusbar",false);

    // Font
    FXString fontspec;
    fontspec=editor->getFont()->getFont();
        getApp()->reg().writeStringEntry("FXTxtView","textfont",fontspec.text());

	// Write registry settings
	getApp()->reg().write();
}


// Quit
long TextEncryptor::onCmdQuit(FXObject* sender,FXSelector sel,void* ptr){
      // ask for saving changes on the current document
    if(isModified()){
      if(!saveChanges())
        getApp()->beep();
    }
    // Save settings
    saveConfig();

    // Quit
    getApp()->exit(0);
    return 1;
}


/*******************************************************************************/

// Text inserted; callback has [pos nins]
long TextEncryptor::onTextInserted(FXObject*,FXSelector,void* ptr){
  const FXTextChange *change=(const FXTextChange*)ptr;

  FXTRACE((140,"Inserted: pos=%d ndel=%d nins=%d\n",change->pos,change->ndel,change->nins));

  // Log undo record
  if(!undolist.busy()){
    undolist.add(new FXTextInsert(editor,change->pos,change->nins,change->ins));
    if(undolist.size()>MAXUNDOSIZE) undolist.trimSize(KEEPUNDOSIZE);
  }

  return 1;
}


// Text deleted; callback has [pos ndel]
long TextEncryptor::onTextDeleted(FXObject*,FXSelector,void* ptr){
  const FXTextChange *change=(const FXTextChange*)ptr;

  FXTRACE((140,"Deleted: pos=%d ndel=%d nins=%d\n",change->pos,change->ndel,change->nins));

  // Log undo record
  if(!undolist.busy()){
    undolist.add(new FXTextDelete(editor,change->pos,change->ndel,change->del));
    if(undolist.size()>MAXUNDOSIZE) undolist.trimSize(KEEPUNDOSIZE);
  }

  return 1;
}


// Text replaced; callback has [pos ndel nins]
long TextEncryptor::onTextReplaced(FXObject*,FXSelector,void* ptr){
  const FXTextChange *change=(const FXTextChange*)ptr;

  FXTRACE((140,"Replaced: pos=%d ndel=%d nins=%d\n",change->pos,change->ndel,change->nins));

  // Log undo record
  if(!undolist.busy()){
    undolist.add(new FXTextReplace(editor,change->pos,change->ndel,change->nins,change->del,change->ins));
    if(undolist.size()>MAXUNDOSIZE) undolist.trimSize(KEEPUNDOSIZE);
  }

  return 1;
}


// Text cursor pos changed
long TextEncryptor::onTextChanged(FXObject*,FXSelector,void* ptr){
    FXTRACE((1,"onTextChanged cursorpos=%d\n",(FXint)(FXival)ptr));
    return 1;
}


// Released right button
long TextEncryptor::onTextRightMouse(FXObject*,FXSelector,void* ptr){
  FXEvent* event=(FXEvent*)ptr;
  if(!event->moved){
    popupmenu->popup(NULL,event->root_x,event->root_y);
    getApp()->runModalWhileShown(popupmenu);
  }
  return 1;
}

//******************************************************************************/

// Update title
long TextEncryptor::onUpdTitle(FXObject* sender,FXSelector,void*)
{
    FXString title="Text Viewer: " + filename;
    sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_SETSTRINGVALUE),(void*)&title);
    return 1;
}


// Save Update
long TextEncryptor::onUpdSave(FXObject* sender,FXSelector,void*)
{
    sender->handle(this,isModified()?FXSEL(SEL_COMMAND,FXWindow::ID_ENABLE):FXSEL(SEL_COMMAND,FXWindow::ID_DISABLE),NULL);
    return 1;
}


// Update print, save and find menus
long TextEncryptor::onUpdMenu(FXObject* sender,FXSelector,void*)
{
    if (editor->getLength()==0)
        sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_DISABLE),NULL);
	else
        sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_ENABLE),NULL);
    return 1;
}


// Open find dialog
long TextEncryptor::onCmdFind(FXObject* obj,FXSelector sel,void* ptr)
{
    int loc;
    static FXString str="";

    if(FXSELTYPE(sel)==SEL_UPDATE)
    {
        if(str=="")
            ((FXWindow*)obj)->disable();
        else
            ((FXWindow*)obj)->enable();
        return 1;
    }

    if(FXSELID(sel)==ID_FIND)
    {
        FXDialogBox dialog(this,"Find",DECOR_TITLE|DECOR_BORDER);
        FXVerticalFrame* content=new FXVerticalFrame(&dialog,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,10,10,10,10,10,10);
        text=new FXTextField(content,30,NULL,0,FRAME_SUNKEN|LAYOUT_FILL_X);
        new FXHorizontalSeparator(content,SEPARATOR_GROOVE|LAYOUT_FILL_X);
        FXHorizontalFrame* buttons=new FXHorizontalFrame(content,LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,0,0,0,0,0,0,0,0);
        new FXButton(buttons,"&Cancel",NULL,&dialog,FXDialogBox::ID_CANCEL,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT,0,0,0,0,20,20);
        new FXButton(buttons,"&Find",NULL,&dialog,FXDialogBox::ID_ACCEPT,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT);

        text->setText(str);
        text->setSelection(0,str.length());

        if(text->canFocus())
            text->killFocus();
//           text->setFocus();

        if(!dialog.execute())
            return 0;

        str=text->getText();
    }

    loc=find(str,editor->getCursorPos());
    if(loc==-1)
        FXMessageBox::information(this,MBOX_OK,"Not Found","String '%s' not found",str.text());
    else
    {
        editor->setCursorPos(loc+str.length());
        editor->setTopLine(loc);
        editor->setSelection(loc,str.length());
        editor->setFocus();
    }
    return 1;
}


// Find string
FXint TextEncryptor::find(const FXString str,FXint pos)
{
    FXString buf=editor->getText();
    const char* text=buf.text();
    const char *p = strstr(&text[pos],str.text());
    if(!p)
        return -1;
    return (p-text);
}


// Create and show window
void TextEncryptor::create()
{
    getApp()->setForeColor(theme_fxmmfore);
    getApp()->setBaseColor(theme_fxmmbase);
    getApp()->setBackColor(theme_fxmmback);
    getApp()->setBorderColor(theme_fxmmborder);
    getApp()->setSelMenuTextColor(theme_fxmmmenufore);
    getApp()->setSelMenuBackColor(theme_fxmmmenuback);
    getApp()->setSelforeColor(theme_fxmmselfore);
    getApp()->setSelbackColor(theme_fxmmselback);
    getApp()->setTipforeColor(theme_fxmmtipfore);
    getApp()->setTipbackColor(theme_fxmmtipback);
    getApp()->setHiliteColor(theme_fxmmhilite);
    getApp()->setShadowColor(theme_fxmmshadow);

    // Default text font
    textfont=NULL;
    fontspec_t=getApp()->reg().readStringEntry("FXTxtView","textfont","");
    if(!fontspec_t.empty()){
      textfont=new FXFont(getApp(),fontspec_t);
      editor->setFont(textfont);
    }else{
      // get the size of normal font and create a new font
      getApp()->getNormalFont()->create();
#ifdef HAVE_LIBFOX_1_7
      FXFontDesc fontdescription=getApp()->getNormalFont()->getFontDesc();
#elif HAVE_LIBFOX_1_6
      FXFontDesc fontdescription;
      getApp()->getNormalFont()->getFontDesc(fontdescription);
#endif
      textfont=new FXFont(getApp(),fontdescription);
      textfont->create();
    }

    fontspec_m=getApp()->reg().readStringEntry("SETTINGS","menufont","");
    if(!fontspec_m.empty()){
      menufont=new FXFont(getApp(),fontspec_m);
      getApp()->setNormalFont(menufont);


    title_file->setFont(menufont);
    title_edit->setFont(menufont);
    title_search->setFont(menufont);
    title_pref->setFont(menufont);
    title_crypt->setFont(menufont);
    title_help->setFont(menufont);
    /*****************************/

    check_tabbook->setFont(menufont);
    check_menubar->setFont(menufont);
    check_toolbar->setFont(menufont);
    check_statusbar->setFont(menufont);

    button_new->setFont(menufont);
    button_open->setFont(menufont);
    button_save->setFont(menufont);
    button_saveas->setFont(menufont);
    button_print->setFont(menufont);
    button_quit->setFont(menufont);

    button_undo->setFont(menufont);
    button_redo->setFont(menufont);

    button_cut->setFont(menufont);
    button_copy->setFont(menufont);
    button_paste->setFont(menufont);
    button_delete->setFont(menufont);

    button_find->setFont(menufont);
    button_finda->setFont(menufont);
    button_setkey->setFont(menufont);
    button_encrypt->setFont(menufont);
    button_decrypt->setFont(menufont);

    button_font->setFont(menufont);
    button_about->setFont(menufont);
    button_help->setFont(menufont);


    cmd_new->setFont(menufont);
    cmd_open->setFont(menufont);
    cmd_save->setFont(menufont);
    cmd_savea->setFont(menufont);
    cmd_print->setFont(menufont);
    cmd_quit->setFont(menufont);
    cmd_undo->setFont(menufont);
    cmd_redo->setFont(menufont);
    cmd_cut->setFont(menufont);
    cmd_copy->setFont(menufont);
    cmd_paste->setFont(menufont);
    cmd_delete->setFont(menufont);
    cmd_pcut->setFont(menufont);
    cmd_pcopy->setFont(menufont);
    cmd_ppaste->setFont(menufont);
    cmd_pdelete->setFont(menufont);
    cmd_all->setFont(menufont);
    cmd_find->setFont(menufont);
    cmd_finda->setFont(menufont);
    cmd_font->setFont(menufont);
    cmd_key->setFont(menufont);
    cmd_encrypt->setFont(menufont);
    cmd_decrypt->setFont(menufont);
    cmd_about->setFont(menufont);
    cmd_help->setFont(menufont);
    /*****************************/
    }

   // Get tabbok status
    if(getApp()->reg().readBoolEntry("FXTxtView","showtabbook",true)==false)
        tabbook->hide();

   // Get menubar status
    if(getApp()->reg().readBoolEntry("FXTxtView","showmenubar",true)==false)
        menubar->hide();

   // Get toolbar status
    if(getApp()->reg().readBoolEntry("FXTxtView","showtoolbar",true)==false)
        toolbar->hide();

    // Get status bar status
    if(getApp()->reg().readBoolEntry("FXTxtView","showstatusbar",true)==false)
        statusbar->hide();

    FXMainWindow::create();
    show();
    editor->setFocus();
    editor->handle(this,FXSEL(SEL_COMMAND,FXText::ID_TOGGLE_EDITABLE),NULL);
}


// read configuration
FXbool TextEncryptor::readConfig() {
    // Read registry
	if(!getApp()->reg().read()){
		return false;
	}

    // Wheel scrolling
    FXint wheellines=getApp()->reg().readIntEntry("SETTINGS","fxmmwheellines",5);
    getApp()->setWheelLines(wheellines);

    // Get size and position
    FXint width,height,xpos,ypos;
    xpos=getApp()->reg().readIntEntry("FXTxtView","xpos",40);
    ypos=getApp()->reg().readIntEntry("FXTxtView","ypos",40);
    width=getApp()->reg().readIntEntry("FXTxtView","width",640);
    height=getApp()->reg().readIntEntry("FXTxtView","height",400);
    // Reposition window
    setX(xpos);
    setY(ypos);
    setWidth(width);
    setHeight(height);
    // Reposition window
    //position(xpos,ypos,width,height);

    // Read Colors
    if(! (theme_fxmmfore=getApp()->reg().readColorEntry("SETTINGS","fxmmforecolor"))) { theme_fxmmfore   = FXRGB(180,180,180);}
    if(! (theme_fxmmback=getApp()->reg().readColorEntry("SETTINGS","fxmmbackcolor"))) { theme_fxmmback   = FXRGB( 20, 20, 20);}
    if(! (theme_fxmmbase=getApp()->reg().readColorEntry("SETTINGS","fxmmbasecolor"))) { theme_fxmmbase   = FXRGB( 60, 60, 60);}
    if(! (theme_fxmmborder=getApp()->reg().readColorEntry("SETTINGS","fxmmbordercolor"))) { theme_fxmmborder = FXRGB(  0,  0,  0);}
    if(! (theme_fxmmmenufore=getApp()->reg().readColorEntry("SETTINGS","fxmmmenuforecolor"))) { theme_fxmmmenufore  = FXRGB(240,240,240);}
    if(! (theme_fxmmmenuback=getApp()->reg().readColorEntry("SETTINGS","fxmmmenubackcolor"))) { theme_fxmmmenuback  = FXRGB( 10, 10, 10);}
    if(! (theme_fxmmselfore=getApp()->reg().readColorEntry("SETTINGS","fxmmselforecolor"))) { theme_fxmmselfore   = FXRGB(  0,  0,  0);}
    if(! (theme_fxmmselback=getApp()->reg().readColorEntry("SETTINGS","fxmmselbackcolor"))) { theme_fxmmselback   = FXRGB(140,140,140);}
    if(! (theme_fxmmtipfore=getApp()->reg().readColorEntry("SETTINGS","fxmmtipforecolor"))) { theme_fxmmtipfore  = FXRGB(220,220,220);}
    if(! (theme_fxmmtipback=getApp()->reg().readColorEntry("SETTINGS","fxmmtipbackcolor"))) { theme_fxmmtipback  = FXRGB( 20, 20, 20);}
    if(! (theme_fxmmhilite=getApp()->reg().readColorEntry("SETTINGS","fxmmhilitecolor"))) { theme_fxmmhilite  = FXRGB(100,100,100);}
    if(! (theme_fxmmshadow=getApp()->reg().readColorEntry("SETTINGS","fxmmshadowcolor"))) { theme_fxmmshadow  = FXRGB( 10, 10, 10);}

  language=getApp()->reg().readStringEntry("SETTINGS","language");
  if(language== "en" || language== "it" || language== "fr" || language== "de" || language== "es" || language== "ru" || language== "cn" || language== "jp" ){
  // Read language
    menu_op=getApp()->reg().readStringEntry("LANG_MENU","menu_op");
    menu_cu=getApp()->reg().readStringEntry("LANG_MENU","menu_cu");
    menu_co=getApp()->reg().readStringEntry("LANG_MENU","menu_co");
    menu_pa=getApp()->reg().readStringEntry("LANG_MENU","menu_pa");
    menu_de=getApp()->reg().readStringEntry("LANG_MENU","menu_de");
    menu_al=getApp()->reg().readStringEntry("LANG_MENU","menu_al");

    menu_ne=getApp()->reg().readStringEntry("LANG_MENU","menu_ne");
    menu_ss=getApp()->reg().readStringEntry("LANG_MENU","menu_ss");
    menu_sa=getApp()->reg().readStringEntry("LANG_MENU","menu_sa");
    menu_pd=getApp()->reg().readStringEntry("LANG_MENU","menu_pd");
    menu_qu=getApp()->reg().readStringEntry("LANG_MENU","menu_qu");
    menu_ff=getApp()->reg().readStringEntry("LANG_MENU","menu_ff");
    menu_fa=getApp()->reg().readStringEntry("LANG_MENU","menu_fa");
    menu_fo=getApp()->reg().readStringEntry("LANG_MENU","menu_fo");
    menu_cf=getApp()->reg().readStringEntry("LANG_MENU","menu_cf");
    menu_en=getApp()->reg().readStringEntry("LANG_MENU","menu_en");

  } else if(language=="cu"){
  // Read custom language
    menu_op=getApp()->reg().readStringEntry("LANG_MENU_CUSTOM","menu_op");
    menu_cu=getApp()->reg().readStringEntry("LANG_MENU_CUSTOM","menu_cu");
    menu_co=getApp()->reg().readStringEntry("LANG_MENU_CUSTOM","menu_co");
    menu_pa=getApp()->reg().readStringEntry("LANG_MENU_CUSTOM","menu_pa");
    menu_de=getApp()->reg().readStringEntry("LANG_MENU_CUSTOM","menu_de");
    menu_al=getApp()->reg().readStringEntry("LANG_MENU_CUSTOM","menu_al");

    menu_ne=getApp()->reg().readStringEntry("LANG_MENU_CUSTOM","menu_ne");
    menu_ss=getApp()->reg().readStringEntry("LANG_MENU_CUSTOM","menu_ss");
    menu_sa=getApp()->reg().readStringEntry("LANG_MENU_CUSTOM","menu_sa");
    menu_pd=getApp()->reg().readStringEntry("LANG_MENU_CUSTOM","menu_pd");
    menu_qu=getApp()->reg().readStringEntry("LANG_MENU_CUSTOM","menu_qu");
    menu_ff=getApp()->reg().readStringEntry("LANG_MENU_CUSTOM","menu_ff");
    menu_fa=getApp()->reg().readStringEntry("LANG_MENU_CUSTOM","menu_fa");
    menu_fo=getApp()->reg().readStringEntry("LANG_MENU_CUSTOM","menu_fo");
    menu_cf=getApp()->reg().readStringEntry("LANG_MENU_CUSTOM","menu_cf");
    menu_en=getApp()->reg().readStringEntry("LANG_MENU_CUSTOM","menu_en");
  }
   // Read registry settings
    if(getApp()->reg().read()){
        return true;
    }
}


// Change font
long TextEncryptor::onCmdFont(FXObject*,FXSelector,void*)
{
  FXFontDialog fontdlg(this,"Change Text Font",DECOR_BORDER|DECOR_TITLE);
#ifdef HAVE_LIBFOX_1_7
  FXFontDesc fontdesc=editor->getFont()->getFontDesc();
  fontdlg.setFontDesc(fontdesc);
  if(fontdlg.execute()){
    FXFont *oldfont=textfont;
    fontdesc=fontdlg.getFontDesc();
#elif HAVE_LIBFOX_1_6
  FXFontDesc fontdesc;
  editor->getFont()->getFontDesc(fontdesc);
  fontdlg.setFontSelection(fontdesc);
  if(fontdlg.execute()){
    FXFont *oldfont=textfont;
    fontdlg.getFontSelection(fontdesc);
#endif
    textfont=new FXFont(getApp(),fontdesc);
    textfont->create();
    editor->setFont(textfont);
    delete oldfont;
    }
  return 1;
}


// About box
long TextEncryptor::onCmdAbout(FXObject*,FXSelector,void*)
{
    FXMessageBox about(this,"About FXTxtView",
		"For X Text Viewer (fxtxtview) is a simple text Editor.\n"
		"Copyright (C) 2008-2015 Luigi Ziviani (ziviani_luigi@yahoo.it)\n\n"
		"This program is free software; you can redistribute it\n"
		"and/or modify it under the terms of the GNU General Public\n"
		"License as published by the Free Software Foundation;\n"
		"either version 3 of the License, or (at your option)\n"
		"any later version.\n\n"
		"This program is distributed in the hope that it will be useful,\n"
		"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
		"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"
		"See the GNU General Public License for more details.\n\n"
		"You should have received a copy of the GNU General Public License\n"
		"along with this program; if not, write to the \n"
		"Free Software Foundation, Inc.,\n"
		"59 Temple Place, Suite 330, Boston, MA  02111-1307  USA",
		logoicon,MBOX_OK|DECOR_TITLE|DECOR_BORDER);
    about.execute();
    return 1;
}


// Help box
long TextEncryptor::onCmdHelp(FXObject*,FXSelector,void*)
{
    FXMessageBox help(this,"Use FXTxtView","FXTxtView is based on an early version of AdIE (Copyright Jeroen van der Zijp).\n\n"
            "It includes a simple simmetic algorythm to encrypt text files (with a text key):\n" 
            "a polyalphabetic substitution, according to the method used by Blaise de Vigenère.\n"
            "The text key can be loaded from a text file or written with the keyboard.\n\n"
            "Now a day, this algorythm is too simple to be used for encripting in the real world,\n"
            "It has been included just for educational purpose\n\n"
			"Go to Utilities\n"            
			"1) SET A KEY: Click Configure and input your key\n"
            "2) ENCRYPT: Click the Encrypt button\n"
            "(if not set, a dialog will appear and ask for the key to be used)\n"
            "3) DECRYPT: Click the Decrypt button\n"
            "(if not set, a dialog will appear and ask for the key to be used)\n\n"
            "Have a fun.",
             buildicon,MBOX_OK|DECOR_TITLE|DECOR_BORDER);
    help.execute();
    return 1;
}
