/********************************************************************************
*                                                                               *
*                      F X   M O V I E   M A N A G E R                          *
*                                                                               *
*********************************************************************************
* Copyright(C) 2008-2015 by Luigi Ziviani <ziviani_luigi@yahoo.it>.             *
* All Rights Reserved.                                 v 6.7                    *
*********************************************************************************
* Id: fxplayer2_main.c,                                2014/01/01               *
*********************************************************************************
* This file is part of FXMOVIEMANAGER                                           *
*                                                                               *
* FXMOVIEMANAGER is free software; you can redistribute it and/or modify        *
* it under the terms of the GNU General Public License as published by          *
* the Free Software Foundation; either version 3 of the License, or             *
* (at your option) any later version.                                           *
*                                                                               *
* FXMOVIEMANAGER is distributed in the hope that it will be useful,             *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 *
* GNU General Public License for more details.                                  *
*                                                                               *
* You should have received a copy of the GNU General Public License             *
* along with this program; if not, write to the Free Software                   *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA     * 
********************************************************************************/

#include "config.h"

#ifdef ENABLE_PLAYERS // at least one

#ifdef HAVE_INTTYPES_H
 #include <inttypes.h>
#endif

#include <math.h>
#include <limits.h>
#include <signal.h>
#include <assert.h>  // xcb_xlib
#include <stdint.h>
#include <stdio.h>  

#if HAVE_LIBAVCODEC
 #include <libavcodec/avcodec.h>
#endif
#ifdef HAVE_LIBAVFORMAT
 #include <libavformat/avformat.h>
#endif
#if HAVE_LIBSWSCALE
 #include <libswscale/swscale.h>
#endif

#if HAVE_LIBAVUTIL
 #include <libavutil/pixfmt.h>
#endif

#if HAVE_LIBSDL2
#include <SDL2/SDL.h>
#include <SDL2/SDL_thread.h>

#define SDL_AUDIO_BUFFER_SIZE           1024

typedef struct AudioRegister {
  int             freq;
  int             channels;
  int64_t         channel_layout;
  enum            AVSampleFormat fmt;
} AudioRegister;

typedef struct MediaRegister {

  AVFormatContext *pFormatCtx;
  int              index_video; 
  int              index_audio; 
  char             filename[1024];

  AVFrame         *frame;

  AVStream        *audio_st;
  struct           AudioRegister audio_src;
  struct           AudioRegister audio_tgt;

  uint8_t          silence_buf[SDL_AUDIO_BUFFER_SIZE];
  uint8_t          *audio_buf;
  uint8_t          *audio_buf1;
  int              audio_want_size; 
  unsigned int     audio_buf_size;      ///< in bytes
  unsigned int     audio_buf1_size;      ///< in bytes
  int              audio_buf_index;     ///< in bytes

  AVPacket        audio_pkt;
  AVPacket        audio_pkt_tmp;

  struct          SwrContext *swr_ctx;

} MediaRegister;


SDL_Window*     sdlWindow;
SDL_Renderer*   sdlRenderer;
SDL_Texture*    sdlTexture;

SDL_Rect        sdlRect;  

static int      disable_video = 0;
static int      disable_audio = 0;
static int      full = 0;



static
void closeStream(MediaRegister *mr, int stream_index){
    AVFormatContext *pFormatCtx = mr->pFormatCtx;
    AVCodecContext *pCodecCtx;

    if (stream_index < 0 || stream_index >= pFormatCtx->nb_streams)
        return;
    pCodecCtx = pFormatCtx->streams[stream_index]->codec;

    switch(pCodecCtx->codec_type) {
    case AVMEDIA_TYPE_AUDIO:
        SDL_CloseAudio();
        av_free_packet(&mr->audio_pkt);
        swr_free(&mr->swr_ctx);
        av_freep(&mr->audio_buf1);
        mr->audio_buf1_size = 0;
        mr->audio_buf = NULL;
        av_frame_free(&mr->frame);
        break;
    case AVMEDIA_TYPE_VIDEO:
      if(sdlTexture)
        SDL_DestroyTexture(sdlTexture);  

      if(sdlWindow)
        SDL_DestroyWindow(sdlWindow);  

        break;
    default:
        break;
    }
    // Close the codec  
    avcodec_close(pCodecCtx);
}


static int audio_decode_frame(MediaRegister *mr) {
fprintf(stderr, "-->audio_decode_frame\n");
/*
    AVFormatContext *pFormatCtx = mr->pFormatCtx;
    AVCodecContext *pCodecCtx;
    // Get a pointer to the codec context for the video stream 
    pCodecCtx = pFormatCtx->streams[mr->index_audio]->codec;  

    //Allocate video frame  
    AVFrame *pFrame = av_frame_alloc();  
    AVPacket packet;

    while( av_read_frame(pFormatCtx, &packet) >= 0 ) { 
        // Is this a packet from the audio stream?  
        if( packet.stream_index == mr->index_audio ) { 

            // Decode audio frame  
            int frameFinished;  

            if(avcodec_decode_audio4(pCodecCtx, pFrame, &frameFinished, &packet)) {

                // Did we get an audio frame?  
                if( frameFinished ) {  
                    AVRational tb = (AVRational){1, pFrame->sample_rate};
                    if (pFrame->pts != AV_NOPTS_VALUE)
                        pFrame->pts = av_rescale_q(pFrame->pts, pCodecCtx->time_base, tb);
                }


            }
        }


    } 
 
    av_frame_free(&pFrame);
*/

  AVPacket *pkt = &mr->audio_pkt;
  AVPacket *pkt_tmp = &mr->audio_pkt_tmp;

  AVFormatContext *pFormatCtx = mr->pFormatCtx;
  AVCodecContext *dec = pFormatCtx->streams[mr->index_audio]->codec; 

  int64_t dec_channel_layout;
  int or_data_size, cv_data_size, len_a, len_b;
  int new_pkt = 0;
  int flush_done = 0;
  int got_frame;
  int wanted_nb_samples;
  AVRational tb;
  int ret;
  int reconfigure;

  for(;;) {

    // an audio packet can contain several frames
    while(pkt_tmp->size > 0 || (!pkt_tmp->data && new_pkt) ){
        fprintf(stderr, "1pkt_tmp->data %d\n", pkt_tmp->data);
        fprintf(stderr, "1pkt_tmp->size %d\n", pkt_tmp->size);
      if (!mr->frame) {
        fprintf(stderr, "!mr->frame\n");
        if (!(mr->frame = av_frame_alloc()))
          return AVERROR(ENOMEM);
      } else
        av_frame_unref(mr->frame);

        new_pkt = 0;
        len_a = avcodec_decode_audio4(dec, mr->frame, &got_frame, pkt_tmp); 
        // if error, skip the frame
        if(len_a < 0) {
	      pkt_tmp->size = 0;
          fprintf(stderr, "break: pkt_tmp->size %d\n", pkt_tmp->size);
	      break;
        }

        pkt_tmp->data += len_a;  // increase
        pkt_tmp->size -= len_a;  // decrease
        fprintf(stderr, "2pkt_tmp->data %d\n", pkt_tmp->data);
        fprintf(stderr, "2pkt_tmp->size %d\n", pkt_tmp->size);

        if(!got_frame) {
          // stop sending empty packets because the decoding is finished
           if (!pkt_tmp->data && dec->codec->capabilities & CODEC_CAP_DELAY)
              flush_done = 1;
           // No data yet, get next frame
	       continue;
        }
        tb = (AVRational){1, mr->frame->sample_rate};

        if(mr->frame->pts != AV_NOPTS_VALUE) // DEBUG (0, AV_NOPTS_VALUE)
           mr->frame->pts = av_rescale_q(mr->frame->pts, dec->time_base, tb);
        else 
        if(mr->frame->pkt_pts != AV_NOPTS_VALUE) 
           mr->frame->pts = av_rescale_q(mr->frame->pkt_pts, mr->audio_st->time_base, tb);

        fprintf(stderr, "mr->frame->pts %d\n", mr->frame->pts);

        if(pkt_tmp->pts != AV_NOPTS_VALUE)
           pkt_tmp->pts += (double)mr->frame->nb_samples / mr->frame->sample_rate / av_q2d(mr->audio_st->time_base);

      }


      // original audio data size
      or_data_size = av_samples_get_buffer_size(NULL, 
                                                av_frame_get_channels(mr->frame),
                                                mr->frame->nb_samples, 
                                                mr->frame->format, 
                                                1);

      dec_channel_layout = (mr->frame->channel_layout && av_frame_get_channels(mr->frame) == av_get_channel_layout_nb_channels(mr->frame->channel_layout)) ? 
                            mr->frame->channel_layout : 
                            av_get_default_channel_layout(av_frame_get_channels(mr->frame)); 

      // need sincronization XXXXXXXXXXXXXX
      wanted_nb_samples = mr->frame->nb_samples;

      // if conversion has not be done yet try to convert
      if (dec_channel_layout     != mr->audio_src.channel_layout || 
          mr->frame->format      != mr->audio_src.fmt            || 
          mr->frame->sample_rate != mr->audio_src.freq           ||
          (wanted_nb_samples     != mr->frame->nb_samples && !mr->swr_ctx)){
        // free the converter buffer
        swr_free(&mr->swr_ctx);
        // try and use the audio converter
        mr->swr_ctx = swr_alloc_set_opts(NULL,
                                         mr->audio_tgt.channel_layout, mr->audio_tgt.fmt, mr->audio_tgt.freq,
                                         dec_channel_layout,           mr->frame->format, mr->frame->sample_rate,
                                         0, 
                                         NULL);
        if (!mr->swr_ctx || swr_init(mr->swr_ctx) < 0) {
            fprintf(stderr, "Unable to convert Audio!\n");
            break; // exit the loop
        }
        // set converted variables
        mr->audio_src.channel_layout = dec_channel_layout;
        mr->audio_src.fmt = mr->frame->format;
        mr->audio_src.freq = mr->frame->sample_rate;
        mr->audio_src.channels = av_frame_get_channels(mr->frame);
      }

      // if conversion was done
      if (mr->swr_ctx) {
        uint8_t **out = &mr->audio_buf1;
        int out_count = (int64_t)wanted_nb_samples * mr->audio_tgt.freq / mr->frame->sample_rate + 256;
        int out_size  = av_samples_get_buffer_size(NULL, mr->audio_tgt.channels, out_count, mr->audio_tgt.fmt, 0);
        const uint8_t **in = (const uint8_t **) mr->frame->extended_data;

        if (out_size < 0) {
          av_log(NULL, AV_LOG_ERROR, "av_samples_get_buffer_size() failed\n");
          break;
        }

        if (wanted_nb_samples != mr->frame->nb_samples) {
          if (swr_set_compensation(mr->swr_ctx, 
                                   (wanted_nb_samples - mr->frame->nb_samples) * mr->audio_tgt.freq / mr->frame->sample_rate,
                                   wanted_nb_samples * mr->audio_tgt.freq / mr->frame->sample_rate)
                                   < 0) {
            av_log(NULL, AV_LOG_ERROR, "Audio compensation failed\n");
            break;
          }
        }

        av_fast_malloc(&mr->audio_buf1, &mr->audio_buf1_size, out_size);
        if (!mr->audio_buf1)
          return AVERROR(ENOMEM);

        // calculate converted audio buffer
        len_b = swr_convert(mr->swr_ctx, 
                            out, 
                            out_count,
                            in, 
                            mr->frame->nb_samples);

        if (len_b < 0) {
          av_log(NULL, AV_LOG_ERROR, "Audio convesion failed\n");
          break; // exit the loop
        }
        if (len_b == out_count) {
          swr_init(mr->swr_ctx);
          av_log(NULL, AV_LOG_WARNING, "Not enough Audio buffer\n");
        }

        // converted audio frame is stored in audio_buf
        mr->audio_buf = mr->audio_buf1;
        cv_data_size = len_b * mr->audio_tgt.channels * av_get_bytes_per_sample(mr->audio_tgt.fmt);
      } else {
        mr->audio_buf = mr->frame->data[0];
        cv_data_size = or_data_size;
      }


      // return converted audio sample size and continue with the next frame of the audio packet
      fprintf(stderr, "converted audio sample size %d\n", cv_data_size);
      return cv_data_size;


    // free the current packet
    if(pkt->data)
      av_free_packet(pkt);
    // set the end to 0 to avoid overlapping
    memset(pkt_tmp, 0, sizeof(*pkt_tmp));

    *pkt_tmp = *pkt;  
   }
/*
  for(;;) {
    // an audio packet can contain several frames
    while(pkt_tmp->size > 0 || (!pkt_tmp->data && new_pkt) || mr->audio_buf_frames_pending){
      if (!mr->frame) {
        if (!(mr->frame = av_frame_alloc()))
          return AVERROR(ENOMEM);
      } else
        av_frame_unref(vs->frame);

      if (vs->audioq.serial != vs->audio_pkt_tmp_serial)
        break;

      if (vs->pause_curr)
        return -1;

      if (!vs->audio_buf_frames_pending) {
        if(flush_done)
          break;

        new_pkt = 0;
        len_a = avcodec_decode_audio4(dec, mr->frame, &got_frame, pkt_tmp); 
        // if error, skip the frame
        if(len_a < 0) {
	      pkt_tmp->size = 0;
	      break;
        }

        pkt_tmp->data += len_a;  // increase
        pkt_tmp->size -= len_a;  // decrease

        if(!got_frame) {
          // stop sending empty packets because the decoding is finished
           if (!pkt_tmp->data && dec->codec->capabilities & CODEC_CAP_DELAY)
              flush_done = 1;
           // No data yet, get next frame
	       continue;
        }

        tb = (AVRational){1, vs->frame->sample_rate};

        if(vs->frame->pts != AV_NOPTS_VALUE) // DEBUG (0, AV_NOPTS_VALUE)
           vs->frame->pts = av_rescale_q(vs->frame->pts, dec->time_base, tb);
        else 
        if(vs->frame->pkt_pts != AV_NOPTS_VALUE) 
           vs->frame->pts = av_rescale_q(vs->frame->pkt_pts, vs->audio_st->time_base, tb);

        if(pkt_tmp->pts != AV_NOPTS_VALUE)
           pkt_tmp->pts += (double)vs->frame->nb_samples / vs->frame->sample_rate / av_q2d(vs->audio_st->time_base);
      }

      // original audio data size
      or_data_size = av_samples_get_buffer_size(NULL, 
                                                av_frame_get_channels(vs->frame),
                                                vs->frame->nb_samples, 
                                                vs->frame->format, 
                                                1);

      dec_channel_layout = (vs->frame->channel_layout && av_frame_get_channels(vs->frame) == av_get_channel_layout_nb_channels(vs->frame->channel_layout)) ? 
                            vs->frame->channel_layout : 
                            av_get_default_channel_layout(av_frame_get_channels(vs->frame)); 

      // audio synchronization
      wanted_nb_samples = synchronize_audio(vs, vs->frame->nb_samples);

      // if conversion has not be done yet try to convert
      if (dec_channel_layout     != vs->audio_src.channel_layout || 
          vs->frame->format      != vs->audio_src.fmt            || 
          vs->frame->sample_rate != vs->audio_src.freq           ||
          (wanted_nb_samples     != vs->frame->nb_samples && !vs->swr_ctx)){
        // free the converter buffer
        swr_free(&vs->swr_ctx);
        // try and use the audio converter
        vs->swr_ctx = swr_alloc_set_opts(NULL,
                                         vs->audio_tgt.channel_layout, vs->audio_tgt.fmt, vs->audio_tgt.freq,
                                         dec_channel_layout,           vs->frame->format, vs->frame->sample_rate,
                                         0, 
                                         NULL);

        // if unable to convert audio skip setting converted variables and print to stdout
        if (!vs->swr_ctx || swr_init(vs->swr_ctx) < 0) {
          av_log(NULL, AV_LOG_ERROR,
                 "Unable to convert FROM sample rate %d Hz %s format %d channels TO sample rate %d Hz %s format %d channels!\n",
          vs->frame->sample_rate,
          av_get_sample_fmt_name(vs->frame->format),
          av_frame_get_channels(vs->frame),
          vs->audio_tgt.freq,
          av_get_sample_fmt_name(vs->audio_tgt.fmt),
          vs->audio_tgt.channels);
          break; // exit the loop
        }
        // set converted variables
        vs->audio_src.channel_layout = dec_channel_layout;
        vs->audio_src.fmt = vs->frame->format;
        vs->audio_src.freq = vs->frame->sample_rate;
        vs->audio_src.channels = av_frame_get_channels(vs->frame);
      }

      // if conversion was done
      if (vs->swr_ctx) {
        uint8_t **out = &vs->audio_buf1;
        int out_count = (int64_t)wanted_nb_samples * vs->audio_tgt.freq / vs->frame->sample_rate + 256;
        int out_size  = av_samples_get_buffer_size(NULL, vs->audio_tgt.channels, out_count, vs->audio_tgt.fmt, 0);
        const uint8_t **in = (const uint8_t **) vs->frame->extended_data;

        if (out_size < 0) {
          av_log(NULL, AV_LOG_ERROR, "av_samples_get_buffer_size() failed\n");
          break;
        }

        if (wanted_nb_samples != vs->frame->nb_samples) {
          if (swr_set_compensation(vs->swr_ctx, (wanted_nb_samples - vs->frame->nb_samples) * vs->audio_tgt.freq / vs->frame->sample_rate,
                                   wanted_nb_samples * vs->audio_tgt.freq / vs->frame->sample_rate) < 0) {
            av_log(NULL, AV_LOG_ERROR, "Audio compensation failed\n");
            break;
          }
        }

        av_fast_malloc(&vs->audio_buf1, &vs->audio_buf1_size, out_size);
        if (!vs->audio_buf1)
          return AVERROR(ENOMEM);

        // calculate converted audio buffer
        len_b = swr_convert(vs->swr_ctx, 
                            out, 
                            out_count,
                            in, 
                            vs->frame->nb_samples);

        if (len_b < 0) {
          av_log(NULL, AV_LOG_ERROR, "Audio convesion failed\n");
          break; // exit the loop
        }
        if (len_b == out_count) {
          swr_init(vs->swr_ctx);
          av_log(NULL, AV_LOG_WARNING, "Not enough Audio buffer\n");
        }

        // converted audio frame is stored in audio_buf
        vs->audio_buf = vs->audio_buf1;
        cv_data_size = len_b * vs->audio_tgt.channels * av_get_bytes_per_sample(vs->audio_tgt.fmt);
      } else {
        vs->audio_buf = vs->frame->data[0];
        cv_data_size = or_data_size;
      }

      // update the audio clock with the pts 
      if (vs->frame->pts != AV_NOPTS_VALUE) // DEBUG (0, AV_NOPTS_VALUE)
        vs->audio_clock = vs->frame->pts * av_q2d(tb) + (double) vs->frame->nb_samples / vs->frame->sample_rate;
      else
        vs->audio_clock = NAN;

      vs->audio_clock_serial = vs->audio_pkt_tmp_serial;

      // return converted audio sample size and continue with the next frame of the audio packet
      return cv_data_size;
    }

    // free the current packet
    if(pkt->data)
      av_free_packet(pkt);
    // set the end to 0 to avoid overlapping
    memset(pkt_tmp, 0, sizeof(*pkt_tmp));

    if(vs->audioq.quit) 
      return -1;

    if (vs->audioq.nb_packets == 0)
      SDL_CondSignal(vs->continue_read_thread);

    // get next packet on quewe
    if ((new_pkt = packet_queue_get(&vs->audioq, pkt, 1, &vs->audio_pkt_tmp_serial)) < 0)
      return -1;

    if(pkt->data == flush_pkt.data) {
      avcodec_flush_buffers(dec);
      flush_done = 0;
      vs->audio_buf_frames_pending = 0;
    }

    *pkt_tmp = *pkt;  
  }
*/
}

// prepare a new SDL audio buffer  (substitute  current_audio_buf)
static void sdl_audio_callback(void *userdata, Uint8 *stream, int len) {
fprintf(stderr, "-->sdl_audio_callback\n");

  MediaRegister *mr = userdata;
  int64_t audio_callback_time;
  int audio_size, len_a;
  int bytes_per_sec, audio_buffer;
  double pts; 
  int frame_size = av_samples_get_buffer_size(NULL, 
                                              mr->audio_tgt.channels, 
                                              1, 
                                              mr->audio_tgt.fmt, 
                                              1);
  audio_callback_time = av_gettime();

  while(len > 0) {

    if(mr->audio_buf_index >= mr->audio_buf_size) {
       audio_size = audio_decode_frame(mr);  // get converted audio sample size
       if(audio_size < 0) {
	      // if error the output silence
	      mr->audio_buf      = mr->silence_buf;
	      mr->audio_buf_size = sizeof(mr->silence_buf) / frame_size * frame_size;
       } else {
	      mr->audio_buf_size = audio_size;
       }
       mr->audio_buf_index = 0;
    }

    len_a = mr->audio_buf_size - mr->audio_buf_index;

    if(len_a > len)
       len_a = len;

    memcpy(stream, (uint8_t *)mr->audio_buf + mr->audio_buf_index, len_a);

    len -= len_a;
    stream += len_a;
    mr->audio_buf_index += len_a;
  }
  // calculate the audio stream bytes per second
  bytes_per_sec = mr->audio_tgt.freq * mr->audio_tgt.channels * av_get_bytes_per_sample(mr->audio_tgt.fmt);
  audio_buffer = mr->audio_buf_size - mr->audio_buf_index;
/*
  // Assume the audio driver that is used by SDL has two periods
  if (!isnan(mr->audio_clock)) {  
    //av_log(NULL, AV_LOG_WARNING, "valid value of audio_clock=%0.3f\n", vs->audio_clock);
    set_clock_at(&mr->audclk, mr->audio_clock - (double)(2 * mr->audio_want_size + audio_buffer) / bytes_per_sec, vs->audio_clock_serial, audio_callback_time / (double)AV_TIME_BASE);
    sync_clock_to_slave(&mr->extclk, &mr->audclk);
  }
*/
}

static 
int audio_open(void *userdata, int64_t want_layout, int want_number, int want_rate, struct AudioRegister *ar) {
fprintf(stderr, "-->audio_open\n");

    SDL_AudioSpec sdl_audio_want;
    SDL_AudioSpec sdl_audio_got;

 SDL_AudioDeviceID id;
 char *device;
 int count;
 int i;

    const char *env;
    const int next_number[] = {0, 0, 1, 6, 2, 6, 4, 6};

    env = SDL_getenv("SDL_AUDIO_CHANNELS");
    if (env) {
        want_number = atoi(env);
        want_layout = av_get_default_channel_layout(want_number);
        fprintf(stderr, "SDL_ENV found!\n");
    }
    if (!want_layout || want_number != av_get_channel_layout_nb_channels(want_layout)) {
        want_layout = av_get_default_channel_layout(want_number);
        want_layout &= ~AV_CH_LAYOUT_STEREO_DOWNMIX;
        fprintf(stderr, "SDL_ENV not found, using layout %s number %i\n", want_layout, want_number);
    }
    fprintf(stderr, "layout %d, number %i\n", want_layout, want_number);

    sdl_audio_want.channels = av_get_channel_layout_nb_channels(want_layout);
    if (sdl_audio_want.channels <= 0)
        fprintf(stderr, "Invalid channel count!\n");
    fprintf(stderr, "sdl_audio_want.channels %i\n", sdl_audio_want.channels);

    sdl_audio_want.freq = want_rate;
    if (sdl_audio_want.freq <= 0 )
        fprintf(stderr, "Invalid sample rate!\n");
    fprintf(stderr, "sdl_audio_want.freq %i\n", sdl_audio_want.freq);

    if (sdl_audio_want.channels <= 0 || sdl_audio_want.freq <= 0 )
        return -1;

    // Get number of devices.
    count = SDL_GetNumAudioDevices(0);
 if (count > 0) {
  for (i = 0; i < count; i++) {
       // Get device name 
       device = (char *)SDL_GetAudioDeviceName(i, 0);

    sdl_audio_want.format   = AUDIO_S16SYS;
    sdl_audio_want.samples  = SDL_AUDIO_BUFFER_SIZE;
    sdl_audio_want.callback = sdl_audio_callback;  // return audio buffer
    sdl_audio_want.userdata = userdata;
    sdl_audio_want.silence  = 0;
    fprintf(stderr, "sdl_audio_want.callback %d\n", sdl_audio_want.callback);
    fprintf(stderr, "sdl_audio_want.userdata %d\n", sdl_audio_want.userdata);

    while (SDL_OpenAudioDevice((const char *)device, 0, &sdl_audio_want, &sdl_audio_got,  SDL_AUDIO_ALLOW_ANY_CHANGE) < 0) {
        av_log(NULL, AV_LOG_WARNING, "SDL_OpenAudio (%d channels): %s\n", sdl_audio_want.channels, SDL_GetError());
        sdl_audio_want.channels = next_number[FFMIN(7, sdl_audio_want.channels)];
        if (!sdl_audio_want.channels) {
            av_log(NULL, AV_LOG_ERROR, "No more channel combinations to try, audio open failed\n");
            return -1;
        }
        want_layout = av_get_default_channel_layout(sdl_audio_want.channels);
    }
    if (sdl_audio_got.format != AUDIO_S16SYS) {
         av_log(NULL, AV_LOG_ERROR, "SDL advised audio format %d is not supported!\n", sdl_audio_got.format);
        return -1;
    }
    // check if desired spec is obtained
    if (sdl_audio_got.channels != sdl_audio_want.channels) {
        want_layout = av_get_default_channel_layout(sdl_audio_got.channels);
        if (!want_layout) {
            av_log(NULL, AV_LOG_ERROR, "SDL advised channel count %d is not supported!\n", sdl_audio_got.channels);
            return -1;
        }
    }
  }
 }
    // store values on register
    ar->fmt            = AV_SAMPLE_FMT_S16;
    ar->freq           = sdl_audio_got.freq;
    ar->channel_layout = want_layout;
    ar->channels       = sdl_audio_got.channels;
    fprintf(stderr, "ar->freq %d\n", ar->freq);
    fprintf(stderr, "ar->channel_layout %d\n", ar->channel_layout);
    fprintf(stderr, "ar->channels %d\n", ar->channels);

    return sdl_audio_got.size;

 SDL_CloseAudioDevice(id);

}



static 
int decodeAudio(MediaRegister *mr, AVCodecContext *pCodecCtx) {
    fprintf(stderr, "-->decodeAudio\n");
    AVFormatContext *pFormatCtx = mr->pFormatCtx;

    int got_audio_size;
    int sample_rate; 
    int channels;
    int64_t channel_layout;

    sample_rate    = pCodecCtx->sample_rate;
    fprintf(stderr, "rate %d\n", sample_rate);
    channels       = pCodecCtx->channels;
    fprintf(stderr, "channels %i\n", channels);
    channel_layout = pCodecCtx->channel_layout;
    fprintf(stderr, "layout %d\n", channel_layout);

    // obtain audio buffer size (sdl_audio_got.size)
    if ((got_audio_size = audio_open(mr, channel_layout, channels, sample_rate, &mr->audio_tgt)) < 0)
      return got_audio_size;

    fprintf(stderr, "got sdl_audio_size %d\n", got_audio_size);

    mr->audio_want_size = got_audio_size;
    mr->audio_src       = mr->audio_tgt;
    mr->audio_buf_size  = 0;
    mr->audio_buf_index = 0;
/*
    // averaging filter for audio syncronization
    mr->audio_diff_avg_coef = exp(log(0.01) / AUDIO_DIFF_AVG_NB);
    mr->audio_diff_avg_count = 0;
    // the audio correction is made only if larger error than this threshold
    // because there isn't enough precise audio fifo
    mr->audio_diff_threshold = 2.0 * vs->audio_hw_buf_size / av_samples_get_buffer_size(NULL, vs->audio_tgt.channels, vs->audio_tgt.freq, vs->audio_tgt.fmt, 1);
*/
    memset(&mr->audio_pkt, 0, sizeof(mr->audio_pkt));
    memset(&mr->audio_pkt_tmp, 0, sizeof(mr->audio_pkt_tmp));

    mr->audio_st = pFormatCtx->streams[mr->index_audio];

/*
    mr->audio_stream = stream_index;

    mr->audio_st = avf_ctx->streams[stream_index];

    packet_queue_start(&mr->audioq);
*/
    // toggle audio play
    SDL_PauseAudio(0);
    fprintf(stderr, "toggled audio!\n");

    SDL_Delay(50);


	//SDL_CloseAudio();

	//SDL_FreeWAV(wav_buffer);

    //av_frame_free(&pFrame);

    return 1;
}



static 
int decodeVideo(MediaRegister *mr, AVCodecContext *pCodecCtx) {

    AVFormatContext *pFormatCtx = mr->pFormatCtx;

	//Source color format
    static enum AVPixelFormat src_fix_fmt = AV_PIX_FMT_YUV420P;
	//Objective color format
    static enum AVPixelFormat dst_fix_fmt = PIX_FMT_BGR24;

    //Allocate video frame  
    AVFrame* pFrame = av_frame_alloc();  
    mr->frame=pFrame; // store on register
    AVFrame* pFrameYUV = av_frame_alloc();  
    if( pFrameYUV == NULL ) {  
        return -1;  
    }  
  
    struct SwsContext* sws_ctx = sws_getContext(  
        pCodecCtx->width,  
        pCodecCtx->height,  
        pCodecCtx->pix_fmt,  
        pCodecCtx->width,  
        pCodecCtx->height,  
        dst_fix_fmt, 
        SWS_BILINEAR,  
        NULL,  
        NULL,  
        NULL);  
  
    int numBytes = avpicture_get_size(  
        dst_fix_fmt, 
        pCodecCtx->width,  
        pCodecCtx->height);  

    uint8_t* buffer = (uint8_t *)av_malloc( numBytes*sizeof(uint8_t) );  
  
    avpicture_fill((AVPicture *)pFrameYUV, buffer, dst_fix_fmt,//PIX_FMT_YUV420P,  
        pCodecCtx->width, pCodecCtx->height);  


    AVPacket packet;  
    SDL_Event event;  
    while( av_read_frame(pFormatCtx, &packet) >= 0 ) { 
 
        // Is this a packet from the video stream?  
        if( packet.stream_index == mr->index_video ) {  
            // Decode video frame  
            int frameFinished;  
            avcodec_decode_video2(pCodecCtx, pFrame, &frameFinished, &packet);  
  
            // Did we get a video frame?  
            if( frameFinished ) {  
                sws_scale(  
                    sws_ctx,  
                    (uint8_t const * const *)pFrame->data,  
                    pFrame->linesize,  
                    0,  
                    pCodecCtx->height,  
                    pFrameYUV->data,  
                    pFrameYUV->linesize  
                    );  
                  
                SDL_UpdateTexture( sdlTexture, &sdlRect, pFrameYUV->data[0], pFrameYUV->linesize[0] );  
                SDL_RenderClear( sdlRenderer );  
                SDL_RenderCopy( sdlRenderer, sdlTexture, &sdlRect, &sdlRect );  
                SDL_RenderPresent( sdlRenderer );  
            }  
            SDL_Delay(50);  
        }  

        // Free the packet that was allocated by av_read_frame  
        av_free_packet(&packet);  

        SDL_PollEvent(&event); 
 
        switch( event.type ) {  
            case SDL_QUIT:  
                SDL_Quit();  
                return 1;  
                break;  
            case SDL_KEYDOWN:
                switch(event.key.keysym.sym) {
                    case SDLK_ESCAPE:
                    case SDLK_q:
                        closeStream(mr, mr->index_audio);
                        closeStream(mr, mr->index_video);
                        return 1;
                        break;
                    default:  
                        break;
                }
            default:  
                break;  
        }  

    }  

    // Free the YUV frame  
    av_free(&pFrame);  
    av_free(&pFrameYUV); 

    return 1; 
}


static 
int openStream(MediaRegister *mr, int stream){

    AVFormatContext *pFormatCtx = mr->pFormatCtx;
    AVCodecContext *pCodecCtx;
    AVCodec* pCodec;

    if(stream < 0 || stream >= pFormatCtx->nb_streams)
        return -1;

    // Get a pointer to the codec context for the video stream 
    pCodecCtx = pFormatCtx->streams[stream]->codec;  

    // Find the decoder for the video stream  
    pCodec = avcodec_find_decoder(pCodecCtx->codec_id);  
    if( pCodec == NULL ) {  
        fprintf(stderr, "Unsupported codec!\n");  
        return -1; // Codec not found  
    }  

    // Open codec  
    AVDictionary* optionsDict = NULL;  
    if( avcodec_open2(pCodecCtx, pCodec, &optionsDict) <0 ) {  
        return -1; // Could not open codec  
    }  


    sdlRect.x = 0;  
    sdlRect.y = 0;  
    sdlRect.w = pCodecCtx->width;  
    sdlRect.h = pCodecCtx->height;  
  

    if( SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_TIMER ) == -1) {  
        fprintf(stderr, "Could not initialize SDL - %s\n", SDL_GetError());  
        exit(1);  
    }  

    SDL_WindowFlags sdlFlags = SDL_WINDOW_SHOWN;
    if(full==1) sdlFlags |= SDL_WINDOW_FULLSCREEN;
    else        sdlFlags |= SDL_WINDOW_RESIZABLE;

    char cap_title[1024]="FXMM: ";
    strncat(cap_title, mr->filename, sizeof(cap_title));
    if( !sdlWindow ) { 
      sdlWindow = SDL_CreateWindow(cap_title,  
        SDL_WINDOWPOS_UNDEFINED,  
        SDL_WINDOWPOS_UNDEFINED,  
        pCodecCtx->width,  pCodecCtx->height,  
        sdlFlags);  
    }
    if( !sdlWindow ) {  
        fprintf(stderr, "SDL: could not set video mode - exiting\n");  
        exit(1);  
    }  
  
    //Originally the third parameter is 0
	if(!sdlRenderer)
      sdlRenderer = SDL_CreateRenderer(sdlWindow, -1, SDL_RENDERER_TARGETTEXTURE | SDL_RENDERER_ACCELERATED | SDL_RENDERER_PRESENTVSYNC);

	if(!sdlTexture){
    sdlTexture = SDL_CreateTexture(  
      sdlRenderer,  
        SDL_PIXELFORMAT_BGR24,//SDL_PIXELFORMAT_YV12,  
        SDL_TEXTUREACCESS_STREAMING,//SDL_TEXTUREACCESS_STATIC,  
        pCodecCtx->width,  
        pCodecCtx->height);  
    }
	if(!sdlTexture)
		return -1;

	SDL_SetTextureBlendMode(sdlTexture,SDL_BLENDMODE_BLEND );

    switch(pCodecCtx->codec_type){
    case AVMEDIA_TYPE_AUDIO:
        decodeAudio(mr, pCodecCtx);
        break;
    case AVMEDIA_TYPE_VIDEO:
        decodeVideo(mr, pCodecCtx);
        break;
    default:
        break;
    }
/*
  if(sdlTexture)
    SDL_DestroyTexture(sdlTexture);  

  if(sdlWindow)
    SDL_DestroyWindow(sdlWindow);  
*/

//    if (mr->audio_st >= 0)
//       closeStream(mr, mr->index_audio);      
 
    //SDL_CloseAudio();
    // Close the codec  
    //avcodec_close(pCodecCtx);
  
    return 1;
}



// parse file and call the GUI loop 
//                (filename,   starttime,  duration,  width, height, loop,  full,  drop,  mute, blind, clock, audiodelay, volume, alsactrl, audiocodec, videocodec, filters)
int loadPlayerFile(char *name, int64_t ts, int64_t td, int w, int h, int l, int f, int d, int a, int v, int c, float ta, int as, int ac, char *acn, char *vcn, char *filter){
  printf("start=%d, duration=%d, width=%i, height=%i, loop=%i, full=%i, drop=%i, mute=%i, blind=%i, clock=%i, audelay=%f, volume=%i, control=%i, audio=%s, video=%s, filter=%s\n", ts, td, w, h, l, f, d, a, v, c, ta, as, ac, acn, vcn, filter);

    MediaRegister mr;

    // disable audio or video
    if(a == 0 || a == 1) disable_audio = a;
    if(v == 0 || v == 1) disable_video = v;

    // Register all formats and codecs  
    av_register_all();  
  
    // Open video file  
    AVFormatContext *pFormatCtx = NULL;  

    // allocate filename memory
    stpncpy(mr.filename, name, sizeof(mr.filename));

    if( avformat_open_input(&pFormatCtx, mr.filename, NULL, NULL) != 0 ) {  
        return -1; // Couldn't open file  
    }  

    // register AVformatContext 
    mr.pFormatCtx = pFormatCtx;

    // Retrieve stream information  
    if( avformat_find_stream_info(pFormatCtx, NULL) <0 ) {  
        return -1; // Couldn't find stream information  
    }  
  
    // Dump information about file onto standard error  
    av_dump_format(pFormatCtx, 0, mr.filename, 0);  

    // Find the first video stream  
    int index_video, index_audio = -1;  
    int i;
    for(i=0; i <pFormatCtx->nb_streams; i++) {  
        if( pFormatCtx->streams[i]->codec->codec_type == AVMEDIA_TYPE_VIDEO ) {  
            index_video = i;  
            break;  
        }  
    }  
    for(i=0; i <pFormatCtx->nb_streams; i++) {  
        if( pFormatCtx->streams[i]->codec->codec_type == AVMEDIA_TYPE_AUDIO ) {  
            index_audio = i;  
            break;  
        }  
    } 

    // Open the video stream
    if(index_video >= 0 && disable_video == 0) {
        av_log(NULL, AV_LOG_INFO, "found Video stream at index %i\n", index_video);
        mr.index_video = index_video;
        openStream(&mr, index_video);
    } 

    // Open the audio stream
    if(index_audio >= 0 && disable_audio == 0) {
        av_log(NULL, AV_LOG_INFO, "found Audio stream at index %i\n", index_audio);
        mr.index_audio = index_audio;
        openStream(&mr, index_audio);
    }

    if(index_video < 0 && index_audio < 0) {
        fprintf(stderr, "Unable to find any stream %s\n", mr.filename);
    } 



    // Close the video file  
    avformat_close_input(&pFormatCtx);  
  
    return 0;
}

#endif /* HAVE_SDL */
#endif /* ENABLE_PLAYER */
